#   Copyright 2019 Red Hat Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import argparse
import ddt
from unittest import mock
import uuid

from openstackclient.tests.unit.identity.v3 import fakes as identity_fakes
from osc_lib import exceptions as osc_exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.api_versions import MAX_VERSION
from manilaclient.common.apiclient import exceptions
from manilaclient.common import cliutils
from manilaclient.osc.v2 import share as osc_shares
from manilaclient.tests.unit.osc import osc_fakes
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestShare(manila_fakes.TestShare):

    def setUp(self):
        super(TestShare, self).setUp()
        self.shares_mock = self.app.client_manager.share.shares
        self.shares_mock.reset_mock()

        self.export_locations_mock = (
            self.app.client_manager.share.share_export_locations
        )
        self.export_locations_mock.reset_mock()

        self.projects_mock = self.app.client_manager.identity.projects
        self.projects_mock.reset_mock()

        self.users_mock = self.app.client_manager.identity.users
        self.users_mock.reset_mock()

        self.snapshots_mock = self.app.client_manager.share.share_snapshots
        self.snapshots_mock.reset_mock()

        self.share_types_mock = self.app.client_manager.share.share_types
        self.share_types_mock.reset_mock()

        self.share_networks_mock = self.app.client_manager.share.share_networks
        self.share_networks_mock.reset_mock()

        self.app.client_manager.share.api_version = api_versions.APIVersion(
            MAX_VERSION)

    def setup_shares_mock(self, count):
        shares = manila_fakes.FakeShare.create_shares(count=count)

        self.shares_mock.get = manila_fakes.FakeShare.get_shares(
            shares,
            0)
        return shares


@ddt.ddt
class TestShareCreate(TestShare):

    def setUp(self):
        super(TestShareCreate, self).setUp()

        self.new_share = manila_fakes.FakeShare.create_one_share(
            attrs={'status': 'available'}
        )
        self.shares_mock.create.return_value = self.new_share

        self.shares_mock.get.return_value = self.new_share
        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot())
        self.snapshots_mock.get.return_value = self.share_snapshot
        self.share_type = manila_fakes.FakeShareType.create_one_sharetype()
        self.share_types_mock.get.return_value = self.share_type

        # Get the command object to test
        self.cmd = osc_shares.CreateShare(self.app, None)

        self.datalist = tuple(self.new_share._info.values())
        self.columns = tuple(self.new_share._info.keys())

    def test_share_create_required_args(self):
        """Verifies required arguments."""

        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            availability_zone=None,
            description=None,
            is_public=False,
            metadata={},
            name=None,
            share_group_id=None,
            share_network=None,
            share_proto=self.new_share.share_proto,
            share_type=self.share_type.id,
            size=self.new_share.size,
            snapshot_id=None,
            scheduler_hints={}
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    def test_share_create_missing_required_arg(self):
        """Verifies missing required arguments."""

        arglist = [
            self.new_share.share_proto,
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto)
        ]
        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_create_metadata(self):
        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
            '--property', 'Manila=zorilla',
            '--property', 'Zorilla=manila'
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id),
            ('property', {'Manila': 'zorilla', 'Zorilla': 'manila'}),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            availability_zone=None,
            description=None,
            is_public=False,
            metadata={'Manila': 'zorilla', 'Zorilla': 'manila'},
            name=None,
            share_group_id=None,
            share_network=None,
            share_proto=self.new_share.share_proto,
            share_type=self.share_type.id,
            size=self.new_share.size,
            snapshot_id=None,
            scheduler_hints={}
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    def test_share_create_scheduler_hints(self):
        """Verifies scheduler hints are parsed correctly."""
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.65")

        shares = self.setup_shares_mock(count=2)
        share1_name = shares[0].name
        share2_name = shares[1].name

        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
            '--scheduler-hint', ('same_host=%s' % share1_name),
            '--scheduler-hint', ('different_host=%s' % share2_name),
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id),
            ('scheduler_hint',
                {'same_host': share1_name, 'different_host': share2_name}),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            availability_zone=None,
            description=None,
            is_public=False,
            metadata={},
            name=None,
            share_group_id=None,
            share_network=None,
            share_proto=self.new_share.share_proto,
            share_type=self.share_type.id,
            size=self.new_share.size,
            snapshot_id=None,
            scheduler_hints={'same_host': shares[0].id,
                             'different_host': shares[1].id},
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    def test_share_create_with_snapshot(self):
        """Verifies create share from snapshot."""

        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--snapshot-id', self.share_snapshot.id

        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('snapshot_id', self.share_snapshot.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch(
                'manilaclient.common.apiclient.utils.find_resource',
                mock.Mock(return_value=self.share_snapshot)):
            columns, data = self.cmd.take_action(parsed_args)

            osc_shares.apiutils.find_resource.assert_called_once_with(
                mock.ANY, self.share_snapshot.id)

        self.shares_mock.create.assert_called_with(
            availability_zone=None,
            description=None,
            is_public=False,
            metadata={},
            name=None,
            share_group_id=None,
            share_network=None,
            share_proto=self.new_share.share_proto,
            share_type=None,
            size=self.new_share.size,
            snapshot_id=self.share_snapshot.id,
            scheduler_hints={}
        )
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    def test_share_create_wait(self):

        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
            '--wait'
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.shares_mock.create.assert_called_with(
            availability_zone=None,
            description=None,
            is_public=False,
            metadata={},
            name=None,
            share_group_id=None,
            share_network=None,
            share_proto=self.new_share.share_proto,
            share_type=self.share_type.id,
            size=self.new_share.size,
            snapshot_id=None,
            scheduler_hints={}
        )

        self.shares_mock.get.assert_called_with(self.new_share.id)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    @mock.patch('manilaclient.osc.v2.share.LOG')
    def test_share_create_wait_error(self, mock_logger):

        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
            '--wait'
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_status', return_value=False):
            columns, data = self.cmd.take_action(parsed_args)

            self.shares_mock.create.assert_called_with(
                availability_zone=None,
                description=None,
                is_public=False,
                metadata={},
                name=None,
                share_group_id=None,
                share_network=None,
                share_proto=self.new_share.share_proto,
                share_type=self.share_type.id,
                size=self.new_share.size,
                snapshot_id=None,
                scheduler_hints={}
            )

            mock_logger.error.assert_called_with(
                "ERROR: Share is in error state.")

            self.shares_mock.get.assert_called_with(self.new_share.id)
            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.datalist, data)

    def test_create_share_with_no_existing_share_type(self):
        arglist = [
            self.new_share.share_proto,
            str(self.new_share.size),
        ]
        verifylist = [
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.share_types_mock.get.side_effect = osc_exceptions.CommandError()

        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    @ddt.data('None', 'NONE', 'none')
    def test_create_share_with_the_name_none(self, name):
        arglist = [
            '--name', name,
            self.new_share.share_proto,
            str(self.new_share.size),
            '--share-type', self.share_type.id,
        ]
        verifylist = [
            ('name', name),
            ('share_proto', self.new_share.share_proto),
            ('size', self.new_share.size),
            ('share_type', self.share_type.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareDelete(TestShare):

    def setUp(self):
        super(TestShareDelete, self).setUp()

        self.shares_mock.delete = mock.Mock()
        self.shares_mock.delete.return_value = None

        # Get the command object to test
        self.cmd = osc_shares.DeleteShare(self.app, None)

    def test_share_delete_one(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            shares[0].name
        ]
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', [shares[0].name])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.shares_mock.delete.assert_called_with(shares[0], None)
        self.assertIsNone(result)

    def test_share_delete_many(self):
        shares = self.setup_shares_mock(count=3)

        arglist = [v.id for v in shares]
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', arglist),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        calls = [mock.call(s, None) for s in shares]
        self.shares_mock.delete.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_share_delete_with_force(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            '--force',
            shares[0].name,
        ]
        verifylist = [
            ('force', True),
            ("share_group", None),
            ('shares', [shares[0].name]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.shares_mock.force_delete.assert_called_once_with(shares[0])
        self.assertIsNone(result)

    def test_share_delete_with_soft(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            '--soft',
            shares[0].name,
        ]
        verifylist = [
            ('soft', True),
            ("share_group", None),
            ('shares', [shares[0].name]),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.shares_mock.soft_delete.assert_called_once_with(shares[0])
        self.assertIsNone(result)

    def test_share_delete_wrong_name(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            shares[0].name + '-wrong-name'
        ]
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', [shares[0].name + '-wrong-name'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        self.assertIsNone(result)

        self.shares_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_delete_no_name(self):
        # self.setup_shares_mock(count=1)

        arglist = []
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', '')
        ]

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_delete_wait(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            shares[0].name,
            '--wait'
        ]
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', [shares[0].name]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=True):
            result = self.cmd.take_action(parsed_args)
            self.shares_mock.delete.assert_called_with(shares[0], None)
            self.shares_mock.get.assert_called_with(shares[0].name)
            self.assertIsNone(result)

    def test_share_delete_wait_error(self):
        shares = self.setup_shares_mock(count=1)

        arglist = [
            shares[0].name,
            '--wait'
        ]
        verifylist = [
            ("force", False),
            ("share_group", None),
            ('shares', [shares[0].name]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=False):
            self.assertRaises(
                osc_exceptions.CommandError,
                self.cmd.take_action,
                parsed_args
            )


class TestShareList(TestShare):

    project = identity_fakes.FakeProject.create_one_project()
    user = identity_fakes.FakeUser.create_one_user()

    columns = [
        'ID',
        'Name',
        'Size',
        'Share Proto',
        'Status',
        'Is Public',
        'Share Type Name',
        'Host',
        'Availability Zone'
    ]

    def setUp(self):
        super(TestShareList, self).setUp()

        self.new_share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.list.return_value = [self.new_share]

        self.users_mock.get.return_value = self.user

        self.projects_mock.get.return_value = self.project

        # Get the command object to test
        self.cmd = osc_shares.ListShare(self.app, None)

    def _get_data(self):
        data = ((
            self.new_share.id,
            self.new_share.name,
            self.new_share.size,
            self.new_share.share_proto,
            self.new_share.status,
            self.new_share.is_public,
            self.new_share.share_type_name,
            self.new_share.host,
            self.new_share.availability_zone,
        ),)
        return data

    def _get_search_opts(self):
        search_opts = {
            'all_tenants': False,
            'is_public': False,
            'metadata': {},
            'extra_specs': {},
            'limit': None,
            'name': None,
            'status': None,
            'host': None,
            'share_server_id': None,
            'share_network_id': None,
            'share_type_id': None,
            'snapshot_id': None,
            'share_group_id': None,
            'project_id': None,
            'user_id': None,
            'offset': None,
            'is_soft_deleted': False,
            'export_location': None,
            'name~': None,
            'description~': None,
        }
        return search_opts

    def test_share_list_no_options(self):
        arglist = []
        verifylist = [
            ('long', False),
            ('all_projects', False),
            ('name', None),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_project(self):
        arglist = [
            '--project', self.project.name,
        ]
        verifylist = [
            ('project', self.project.name),
            ('long', False),
            ('all_projects', False),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['project_id'] = self.project.id
        search_opts['all_tenants'] = True

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_project_domain(self):
        arglist = [
            '--project', self.project.name,
            '--project-domain', self.project.domain_id,
        ]
        verifylist = [
            ('project', self.project.name),
            ('project_domain', self.project.domain_id),
            ('long', False),
            ('all_projects', False),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['project_id'] = self.project.id
        search_opts['all_tenants'] = True

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_user(self):
        arglist = [
            '--user', self.user.name,
        ]
        verifylist = [
            ('user', self.user.name),
            ('long', False),
            ('all_projects', False),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['user_id'] = self.user.id

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )
        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_user_domain(self):
        arglist = [
            '--user', self.user.name,
            '--user-domain', self.user.domain_id,
        ]
        verifylist = [
            ('user', self.user.name),
            ('user_domain', self.user.domain_id),
            ('long', False),
            ('all_projects', False),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['user_id'] = self.user.id

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_name(self):
        arglist = [
            '--name', self.new_share.name,
        ]
        verifylist = [
            ('long', False),
            ('all_projects', False),
            ('name', self.new_share.name),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['name'] = self.new_share.name

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_status(self):
        arglist = [
            '--status', self.new_share.status,
        ]
        verifylist = [
            ('long', False),
            ('all_projects', False),
            ('name', None),
            ('status', self.new_share.status),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['status'] = self.new_share.status

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_all_tenants(self):
        arglist = [
            '--all-projects',
        ]
        verifylist = [
            ('long', False),
            ('all_projects', True),
            ('name', None),
            ('status', None),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['all_tenants'] = True

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_long(self):
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', True),
            ('all_projects', False),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        collist = [
            'ID',
            'Name',
            'Size',
            'Share Protocol',
            'Status',
            'Is Public',
            'Share Type Name',
            'Availability Zone',
            'Description',
            'Share Network ID',
            'Share Server ID',
            'Share Type',
            'Share Group ID',
            'Host',
            'User ID',
            'Project ID',
            'Access Rules Status',
            'Source Snapshot ID',
            'Supports Creating Snapshots',
            'Supports Cloning Snapshots',
            'Supports Mounting snapshots',
            'Supports Reverting to Snapshot',
            'Migration Task Status',
            'Source Share Group Snapshot Member ID',
            'Replication Type',
            'Has Replicas',
            'Created At',
            'Properties',
        ]
        self.assertEqual(collist, cmd_columns)

        data = ((
            self.new_share.id,
            self.new_share.name,
            self.new_share.size,
            self.new_share.share_proto,
            self.new_share.status,
            self.new_share.is_public,
            self.new_share.share_type_name,
            self.new_share.availability_zone,
            self.new_share.description,
            self.new_share.share_network_id,
            self.new_share.share_server_id,
            self.new_share.share_type,
            self.new_share.share_group_id,
            self.new_share.host,
            self.new_share.user_id,
            self.new_share.project_id,
            self.new_share.access_rules_status,
            self.new_share.snapshot_id,
            self.new_share.snapshot_support,
            self.new_share.create_share_from_snapshot_support,
            self.new_share.mount_snapshot_support,
            self.new_share.revert_to_snapshot_support,
            self.new_share.task_state,
            self.new_share.source_share_group_snapshot_member_id,
            self.new_share.replication_type,
            self.new_share.has_replicas,
            self.new_share.created_at,
            self.new_share.metadata
        ),)

        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_with_marker_and_limit(self):
        arglist = [
            "--marker", self.new_share.id,
            "--limit", "2",
        ]
        verifylist = [
            ('long', False),
            ('all_projects', False),
            ('name', None),
            ('status', None),
            ('marker', self.new_share.id),
            ('limit', 2),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        self.assertEqual(self.columns, cmd_columns)

        search_opts = self._get_search_opts()

        search_opts['limit'] = 2
        search_opts['offset'] = self.new_share.id

        data = self._get_data()

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts
        )
        self.assertEqual(data, tuple(cmd_data))

    def test_share_list_negative_limit(self):
        arglist = [
            "--limit", "-2",
        ]
        verifylist = [
            ("limit", -2),
        ]
        self.assertRaises(argparse.ArgumentTypeError, self.check_parser,
                          self.cmd, arglist, verifylist)

    def test_share_list_name_description_filter(self):
        arglist = [
            '--name~', self.new_share.name,
            '--description~', self.new_share.description,
        ]

        verifylist = [
            ('name~', self.new_share.name),
            ('description~', self.new_share.description),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cmd_columns, cmd_data = self.cmd.take_action(parsed_args)

        search_opts = self._get_search_opts()

        search_opts['name~'] = self.new_share.name
        search_opts['description~'] = self.new_share.description

        self.shares_mock.list.assert_called_once_with(
            search_opts=search_opts,
        )

        self.assertEqual(self.columns, cmd_columns)

        data = self._get_data()

        self.assertEqual(data, tuple(cmd_data))

    def test_list_share_soft_deleted_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.60")
        arglist = [
            '--soft-deleted',
        ]
        verifylist = [
            ('soft_deleted', True),
        ]

        search_opts = self._get_search_opts()
        search_opts['is_soft_deleted'] = True

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_list_share_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.35")

        arglist = [
            '--name~', 'Name',
            '--description~', 'Description',
        ]
        verifylist = [
            ('name~', 'Name'),
            ('description~', 'Description'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareShow(TestShare):

    def setUp(self):
        super(TestShareShow, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self._share

        self._export_location = (
            manila_fakes.FakeShareExportLocation.create_one_export_location())

        # Get the command object to test
        self.cmd = osc_shares.ShowShare(self.app, None)

    def test_share_show(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ("share", self._share.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        cliutils.convert_dict_list_to_string = mock.Mock()
        cliutils.convert_dict_list_to_string.return_value = dict(
            self._export_location)
        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.get.assert_called_with(self._share.id)

        self.assertEqual(
            manila_fakes.FakeShare.get_share_columns(self._share),
            columns)

        self.assertEqual(
            manila_fakes.FakeShare.get_share_data(self._share),
            data)


class TestShareSet(TestShare):

    def setUp(self):
        super(TestShareSet, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            methods={"reset_state": None, "reset_task_state": None}
        )
        self.shares_mock.get.return_value = self._share

        # Get the command object to test
        self.cmd = osc_shares.SetShare(self.app, None)

    def test_share_set_property(self):
        arglist = [
            '--property', 'Zorilla=manila',
            self._share.id,
        ]
        verifylist = [
            ('property', {'Zorilla': 'manila'}),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.set_metadata.assert_called_with(
            self._share.id,
            {'Zorilla': 'manila'})

    def test_share_set_name(self):
        new_name = uuid.uuid4().hex
        arglist = [
            '--name', new_name,
            self._share.id,
        ]
        verifylist = [
            ('name', new_name),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            display_name=parsed_args.name)

    def test_share_set_description(self):
        new_description = uuid.uuid4().hex
        arglist = [
            '--description', new_description,
            self._share.id,
        ]
        verifylist = [
            ('description', new_description),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            display_description=parsed_args.description)

    def test_share_set_visibility(self):
        arglist = [
            '--public', 'true',
            self._share.id,
        ]
        verifylist = [
            ('public', 'true'),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            is_public='true')

    def test_share_set_visibility_exception(self):
        arglist = [
            '--public', 'not_a_boolean_value',
            self._share.id,
        ]
        verifylist = [
            ('public', 'not_a_boolean_value'),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            is_public='not_a_boolean_value')

        # '--public' takes only boolean value, would raise a BadRequest
        self.shares_mock.update.side_effect = exceptions.BadRequest()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_set_property_exception(self):
        arglist = [
            '--property', 'key=',
            self._share.id,
        ]
        verifylist = [
            ('property', {'key': ''}),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.set_metadata.assert_called_with(
            self._share.id,
            {'key': ''})

        # '--property' takes key=value arguments
        # missing a value would raise a BadRequest
        self.shares_mock.set_metadata.side_effect = exceptions.BadRequest()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_set_status(self):
        new_status = 'available'
        arglist = [
            self._share.id,
            '--status', new_status
        ]
        verifylist = [
            ('share', self._share.id),
            ('status', new_status)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self._share.reset_state.assert_called_with(new_status)
        self.assertIsNone(result)

    def test_share_set_status_exception(self):
        new_status = 'available'
        arglist = [
            self._share.id,
            '--status', new_status
        ]
        verifylist = [
            ('share', self._share.id),
            ('status', new_status)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self._share.reset_state.side_effect = Exception()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_set_task_state(self):
        new_task_state = 'migration_starting'
        arglist = [
            self._share.id,
            '--task-state', new_task_state
        ]
        verifylist = [
            ('share', self._share.id),
            ('task_state', new_task_state)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self._share.reset_task_state.assert_called_with(new_task_state)
        self.assertIsNone(result)


class TestShareUnset(TestShare):

    def setUp(self):
        super(TestShareUnset, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.UnsetShare(self.app, None)

    def test_share_unset_property(self):
        arglist = [
            '--property', 'Manila',
            self._share.id,
        ]
        verifylist = [
            ('property', ['Manila']),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.delete_metadata.assert_called_with(
            self._share.id,
            parsed_args.property)

    def test_share_unset_name(self):
        arglist = [
            '--name',
            self._share.id,
        ]
        verifylist = [
            ('name', True),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            display_name=None)

    def test_share_unset_description(self):
        arglist = [
            '--description',
            self._share.id,
        ]
        verifylist = [
            ('description', True),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.update.assert_called_with(
            self._share.id,
            display_description=None)

    def test_share_unset_name_exception(self):
        arglist = [
            '--name',
            self._share.id,
        ]
        verifylist = [
            ('name', True),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.shares_mock.update.side_effect = exceptions.BadRequest()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_unset_property_exception(self):
        arglist = [
            '--property', 'Manila',
            self._share.id,
        ]
        verifylist = [
            ('property', ['Manila']),
            ('share', self._share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.cmd.take_action(parsed_args)
        self.shares_mock.delete_metadata.assert_called_with(
            self._share.id,
            parsed_args.property)

        # 404 Not Found would be raised, if property 'Manila' doesn't exist
        self.shares_mock.delete_metadata.side_effect = exceptions.NotFound()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestResizeShare(TestShare):

    def setUp(self):
        super(TestResizeShare, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={'size': 2,
                   'status': 'available'})
        self.shares_mock.get.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.ResizeShare(self.app, None)

    def test_share_shrink(self):
        arglist = [
            self._share.id,
            '1'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 1)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.shares_mock.shrink.assert_called_with(
            self._share,
            1
        )

    def test_share_shrink_exception(self):
        arglist = [
            self._share.id,
            '1'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 1)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.shares_mock.shrink.side_effect = Exception()
        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_share_extend(self):
        arglist = [
            self._share.id,
            '3'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 3)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.shares_mock.extend.assert_called_with(
            self._share,
            3
        )

    def test_share_extend_exception(self):
        arglist = [
            self._share.id,
            '3'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 3)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.shares_mock.extend.side_effect = Exception()
        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_share_resize_already_required_size(self):
        arglist = [
            self._share.id,
            '2'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 2)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            osc_exceptions.CommandError,
            self.cmd.take_action,
            parsed_args
        )

    def test_share_missing_args(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ('share', self._share.id)
        ]

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_resize_wait(self):
        arglist = [
            self._share.id,
            '3',
            '--wait'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 3),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.shares_mock.extend.assert_called_with(
            self._share,
            3
        )
        self.shares_mock.get.assert_called_with(self._share.id)

    def test_share_resize_wait_error(self):
        arglist = [
            self._share.id,
            '3',
            '--wait'
        ]
        verifylist = [
            ('share', self._share.id),
            ('new_size', 3),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_status', return_value=False):
            self.assertRaises(
                osc_exceptions.CommandError,
                self.cmd.take_action,
                parsed_args
            )


class TestAdoptShare(TestShare):

    def setUp(self):
        super(TestAdoptShare, self).setUp()

        self._share_type = manila_fakes.FakeShareType.create_one_sharetype()
        self.share_types_mock.get.return_value = self._share_type

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'status': 'available',
                'share_type': self._share_type.id,
                'share_server_id': 'server-id' + uuid.uuid4().hex})
        self.shares_mock.get.return_value = self._share

        self.shares_mock.manage.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.AdoptShare(self.app, None)

        self.datalist = tuple(self._share._info.values())
        self.columns = tuple(self._share._info.keys())

    def test_share_adopt_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_adopt_required_args(self):
        arglist = [
            'some.host@driver#pool',
            'NFS',
            '10.0.0.1:/example_path'
        ]
        verifylist = [
            ('service_host', 'some.host@driver#pool'),
            ('protocol', 'NFS'),
            ('export_path', '10.0.0.1:/example_path')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.manage.assert_called_with(
            description=None,
            export_path='10.0.0.1:/example_path',
            name=None,
            protocol='NFS',
            service_host='some.host@driver#pool'
        )
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    def test_share_adopt(self):
        arglist = [
            'some.host@driver#pool',
            'NFS',
            '10.0.0.1:/example_path',
            '--name', self._share.id,
            '--description', self._share.description,
            '--share-type', self._share.share_type,
            '--driver-options', 'key1=value1', 'key2=value2',
            '--wait',
            '--public',
            '--share-server-id', self._share.share_server_id

        ]
        verifylist = [
            ('service_host', 'some.host@driver#pool'),
            ('protocol', 'NFS'),
            ('export_path', '10.0.0.1:/example_path'),
            ('name', self._share.id),
            ('description', self._share.description),
            ('share_type', self._share_type.id),
            ('driver_options', ['key1=value1', 'key2=value2']),
            ('wait', True),
            ('public', True),
            ('share_server_id', self._share.share_server_id)

        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.manage.assert_called_with(
            description=self._share.description,
            driver_options={'key1': 'value1', 'key2': 'value2'},
            export_path='10.0.0.1:/example_path',
            name=self._share.id,
            protocol='NFS',
            service_host='some.host@driver#pool',
            share_server_id=self._share.share_server_id,
            share_type=self._share_type.id,
            public=True
        )
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)

    @mock.patch('manilaclient.osc.v2.share.LOG')
    def test_share_adopt_wait_error(self, mock_logger):
        arglist = [
            'some.host@driver#pool',
            'NFS',
            '10.0.0.1:/example_path',
            '--wait'
        ]
        verifylist = [
            ('service_host', 'some.host@driver#pool'),
            ('protocol', 'NFS'),
            ('export_path', '10.0.0.1:/example_path'),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_status', return_value=False):
            columns, data = self.cmd.take_action(parsed_args)
            self.shares_mock.manage.assert_called_with(
                description=None,
                export_path='10.0.0.1:/example_path',
                name=None,
                protocol='NFS',
                service_host='some.host@driver#pool'
            )

            mock_logger.error.assert_called_with(
                "ERROR: Share is in error state.")

            self.shares_mock.get.assert_called_with(self._share.id)
            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.datalist, data)

    def test_share_adopt_visibility_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.7")

        arglist = [
            'some.host@driver#pool',
            'NFS',
            '10.0.0.1:/example_path',
            '--public'

        ]
        verifylist = [
            ('service_host', 'some.host@driver#pool'),
            ('protocol', 'NFS'),
            ('export_path', '10.0.0.1:/example_path'),
            ('public', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_share_adopt_share_server_api_version_exception(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.48")

        arglist = [
            'some.host@driver#pool',
            'NFS',
            '10.0.0.1:/example_path',
            '--share-server-id', self._share.share_server_id

        ]
        verifylist = [
            ('service_host', 'some.host@driver#pool'),
            ('protocol', 'NFS'),
            ('export_path', '10.0.0.1:/example_path'),
            ('share_server_id', self._share.share_server_id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestAbandonShare(TestShare):

    def setUp(self):
        super(TestAbandonShare, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={'status': 'available'})
        self.shares_mock.get.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.AbandonShare(self.app, None)

    def test_share_abandon(self):
        arglist = [
            self._share.id,
        ]
        verifylist = [
            ('share', [self._share.id]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.shares_mock.unmanage.assert_called_with(self._share)
        self.assertIsNone(result)

    def test_share_abandon_wait(self):
        arglist = [
            self._share.id,
            '--wait'
        ]
        verifylist = [
            ('share', [self._share.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=True):
            result = self.cmd.take_action(parsed_args)
            self.shares_mock.unmanage.assert_called_with(self._share)
            self.assertIsNone(result)

    def test_share_abandon_wait_error(self):
        arglist = [
            self._share.id,
            '--wait'
        ]
        verifylist = [
            ('share', [self._share.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=False):
            self.assertRaises(
                osc_exceptions.CommandError,
                self.cmd.take_action,
                parsed_args
            )


class TestShareExportLocationShow(TestShare):

    def setUp(self):
        super(TestShareExportLocationShow, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self._share

        self._export_location = (
            manila_fakes.FakeShareExportLocation.create_one_export_location())

        self.export_locations_mock.get.return_value = (
            self._export_location
        )

        # Get the command object to test
        self.cmd = osc_shares.ShareExportLocationShow(self.app, None)

    def test_share_show_export_location(self):
        arglist = [
            self._share.id,
            self._export_location.fake_uuid
        ]
        verifylist = [
            ('share', self._share.id),
            ('export_location', self._export_location.fake_uuid)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.export_locations_mock.get.assert_called_with(
            share=self._share,
            export_location=self._export_location.fake_uuid
        )

        self.assertEqual(tuple(self._export_location._info.keys()), columns)
        self.assertCountEqual(self._export_location._info.values(), data)


class TestShareExportLocationList(TestShare):

    columns = ['ID', 'Path', 'Preferred']

    def setUp(self):
        super(TestShareExportLocationList, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self._share

        self._export_locations = [
            manila_fakes.FakeShareExportLocation.create_one_export_location()]

        self.export_locations_mock.list.return_value = (
            self._export_locations
        )

        self.values = (oscutils.get_dict_properties(
            e._info, self.columns) for e in self._export_locations)

        # Get the command object to test
        self.cmd = osc_shares.ShareExportLocationList(self.app, None)

    def test_share_list_export_location(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ('share', self._share.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.export_locations_mock.list.assert_called_with(
            share=self._share
        )

        self.assertEqual(self.columns, columns)
        self.assertCountEqual(self.values, data)


class TestShowShareProperties(TestShare):

    properties = {
        'key1': 'value1',
        'key2': 'value2'
    }

    def setUp(self):
        super(TestShowShareProperties, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'metadata': osc_fakes.FakeResource(
                    info=self.properties)
            }
        )
        self.shares_mock.get.return_value = self._share
        self.shares_mock.get_metadata.return_value = self._share.metadata

        # Get the command object to test
        self.cmd = osc_shares.ShowShareProperties(self.app, None)

        self.datalist = tuple(self._share.metadata._info.values())
        self.columns = tuple(self._share.metadata._info.keys())

    def test_share_show_properties(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ("share", self._share.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.shares_mock.get.assert_called_with(self._share.id)
        self.shares_mock.get_metadata.assert_called_with(self._share)

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.datalist, data)


class TestShareRevert(TestShare):

    def setUp(self):
        super(TestShareRevert, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share(
            attrs={'revert_to_snapshot_support': True},
            methods={'revert_to_snapshot': None}
        )
        self.share_snapshot = (
            manila_fakes.FakeShareSnapshot.create_one_snapshot(
                attrs={'share_id': self.share.id}))
        self.shares_mock.get.return_value = self.share
        self.snapshots_mock.get.return_value = self.share_snapshot

        self.cmd = osc_shares.RevertShare(self.app, None)

    def test_share_revert(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.shares_mock.get.assert_called_with(self.share_snapshot.share_id)
        self.share.revert_to_snapshot.assert_called_with(self.share_snapshot)
        self.assertIsNone(result)

    def test_share_revert_exception(self):
        arglist = [
            self.share_snapshot.id
        ]
        verifylist = [
            ('snapshot', self.share_snapshot.id)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.share.revert_to_snapshot.side_effect = Exception()
        self.assertRaises(
            osc_exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareMigrationStart(TestShare):

    def setUp(self):
        super(TestShareMigrationStart, self).setUp()

        self.new_share_type = manila_fakes.FakeShareType.create_one_sharetype()
        self.share_types_mock.get.return_value = self.new_share_type

        self.new_share_network = manila_fakes.FakeShareNetwork \
            .create_one_share_network()
        self.share_networks_mock.get.return_value = self.new_share_network

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'status': 'available'},
            methods={'migration_start': None})
        self.shares_mock.get.return_value = self._share

        self.shares_mock.manage.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.ShareMigrationStart(self.app, None)

    def test_migration_start_with_new_share_type(self):
        """Test with new_share_type"""
        arglist = [
            self._share.id,
            'host@driver#pool',
            '--preserve-metadata', 'False',
            '--preserve-snapshots', 'False',
            '--writable', 'False',
            '--nondisruptive', 'False',
            '--new-share-type', self.new_share_type.id,
            '--force-host-assisted-migration', 'False'

        ]
        verifylist = [
            ('share', self._share.id),
            ('host', 'host@driver#pool'),
            ('preserve_metadata', 'False'),
            ('preserve_snapshots', 'False'),
            ('writable', 'False'),
            ('nondisruptive', 'False'),
            ('new_share_type', self.new_share_type.id),
            ('force_host_assisted_migration', 'False')

        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self._share.migration_start.assert_called_with(
            "host@driver#pool",
            'False',
            'False',
            'False',
            'False',
            'False',
            None,
            self.new_share_type.id
        )
        self.assertIsNone(result)

    def test_migration_start_with_new_share_network(self):
        """Test with new_share_network"""

        arglist = [
            self._share.id,
            'host@driver#pool',
            '--preserve-metadata', 'False',
            '--preserve-snapshots', 'False',
            '--writable', 'False',
            '--nondisruptive', 'False',
            '--new-share-network', self.new_share_network.id,
            '--force-host-assisted-migration', 'False'

        ]
        verifylist = [
            ('share', self._share.id),
            ('host', 'host@driver#pool'),
            ('preserve_metadata', 'False'),
            ('preserve_snapshots', 'False'),
            ('writable', 'False'),
            ('nondisruptive', 'False'),
            ('new_share_network', self.new_share_network.id),
            ('force_host_assisted_migration', 'False')

        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self._share.migration_start.assert_called_with(
            "host@driver#pool",
            'False',
            'False',
            'False',
            'False',
            'False',
            self.new_share_network.id,
            None
        )
        self.assertIsNone(result)


class TestShareMigrationCancel(TestShare):

    def setUp(self):
        super(TestShareMigrationCancel, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'status': 'migrating'},
            methods={'migration_cancel': None})
        self.shares_mock.get.return_value = self._share

        self.shares_mock.manage.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.ShareMigrationCancel(self.app, None)

    def test_migration_cancel(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ('share', self._share.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self._share.migration_cancel.assert_called
        self.assertIsNone(result)


class TestShareMigrationComplete(TestShare):

    def setUp(self):
        super(TestShareMigrationComplete, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'status': 'migrating'},
            methods={'migration_complete': None})
        self.shares_mock.get.return_value = self._share

        self.shares_mock.manage.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.ShareMigrationComplete(self.app, None)

    def test_migration_complete(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ('share', self._share.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self._share.migration_complete.assert_called
        self.assertIsNone(result)


class TestShareMigrationShow(TestShare):

    def setUp(self):
        super(TestShareMigrationShow, self).setUp()

        self._share = manila_fakes.FakeShare.create_one_share(
            attrs={
                'status': 'available',
                'task_state': 'migration_in_progress'},
            methods={'migration_get_progress': ("<Response [200]>",
                     {'total_progress': 0, 'task_state':
                      'migration_in_progress',
                      'details': {}})
                     })
        self.shares_mock.get.return_value = self._share

        self.shares_mock.manage.return_value = self._share

        # Get the command objects to test
        self.cmd = osc_shares.ShareMigrationShow(self.app, None)

    def test_migration_show(self):
        arglist = [
            self._share.id
        ]
        verifylist = [
            ('share', self._share.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self._share.migration_get_progress.assert_called


class TestShareRestore(TestShare):

    def setUp(self):
        super(TestShareRestore, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share(
            methods={'restore': None}
        )
        self.shares_mock.get.return_value = self.share
        self.cmd = osc_shares.RestoreShare(self.app, None)

        def test_share_restore(self):
            arglist = [
                self.share.name
            ]
            verifylist = [
                ('share', [self.share.name])
            ]
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            result = self.cmd.take_action(parsed_args)

            self.shares_mock.get.assert_called_with(self.share)
            self.share.restore.assert_called_with(self.share)
            self.assertIsNone(result)

        def test_share_restore_exception(self):
            arglist = [
                self.share.name
            ]
            verifylist = [
                ('share', [self.share.name])
            ]
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            self.share.restore.side_effect = Exception()
            self.assertRaises(
                osc_exceptions.CommandError, self.cmd.take_action, parsed_args)



# Copyright (c) 2017 Hitachi Data Systems
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from tempest.lib import exceptions as tempest_lib_exc
import testtools

from manilaclient import config
from manilaclient.tests.functional import base
from manilaclient.tests.functional import utils

CONF = config.CONF


@utils.skip_if_microversion_not_supported('2.32')
class SnapshotAccessReadBase(base.BaseTestCase):
    protocol = None

    def setUp(self):
        super(SnapshotAccessReadBase, self).setUp()
        if self.protocol not in CONF.enable_protocols:
            message = "%s tests are disabled." % self.protocol
            raise self.skipException(message)
        self.access_types = CONF.access_types_mapping.get(
            self.protocol, '').split(' ')
        if not self.access_types:
            raise self.skipException("No access types were provided for %s "
                                     "snapshot access tests." % self.protocol)

        self.share = self.create_share(share_protocol=self.protocol,
                                       client=self.get_user_client())
        int_range = range(0, 10)

        self.access_to = {
            'ip': ['99.88.77.%d' % i for i in int_range],
            'user': ['foo_user_%d' % i for i in int_range],
            'cert': ['tenant_%d.example.com' % i for i in int_range],
        }

    def _test_create_list_access_rule_for_snapshot(self, snapshot_id):
        access = []
        access_type = self.access_types[0]

        for i in range(5):
            access_ = self.user_client.snapshot_access_allow(
                snapshot_id, access_type,
                self.access_to[access_type][i])
            access.append(access_)

        return access

    def test_create_list_access_rule_for_snapshot(self):
        snapshot = self.create_snapshot(share=self.share['id'],
                                        client=self.get_user_client(),
                                        cleanup_in_class=False)

        access = self._test_create_list_access_rule_for_snapshot(
            snapshot['id'])

        access_list = self.user_client.list_access(
            snapshot['id'], is_snapshot=True)

        for i in range(5):
            self.assertIn(access[i]['id'],
                          [access_list[j]['id'] for j in range(5)])
            self.assertIn(access[i]['access_type'],
                          [access_list[j]['access_type'] for j in range(5)])
            self.assertIn(access[i]['access_to'],
                          [access_list[j]['access_to'] for j in range(5)])
            self.assertIsNotNone(access_list[i]['access_type'])
            self.assertIsNotNone(access_list[i]['access_to'])

    def test_create_list_access_rule_for_snapshot_select_column(self):
        snapshot = self.create_snapshot(share=self.share['id'],
                                        client=self.get_user_client(),
                                        cleanup_in_class=False)

        self._test_create_list_access_rule_for_snapshot(snapshot['id'])

        access_list = self.user_client.list_access(
            snapshot['id'], columns="access_type,access_to", is_snapshot=True)

        self.assertTrue(any(x['Access_Type'] is not None for x in access_list))
        self.assertTrue(any(x['Access_To'] is not None for x in access_list))

    def _create_delete_access_rule(self, snapshot_id, access_type, access_to):

        if access_type not in self.access_types:
            raise self.skipException(
                "'%(access_type)s' access rules is disabled for protocol "
                "'%(protocol)s'." % {"access_type": access_type,
                                     "protocol": self.protocol})

        access = self.user_client.snapshot_access_allow(
            snapshot_id, access_type, access_to)

        self.assertEqual(access_type, access.get('access_type'))
        self.assertEqual(access_to.replace('\\\\', '\\'),
                         access.get('access_to'))

        self.user_client.wait_for_access_rule_status(
            snapshot_id, access['id'], is_snapshot=True)
        self.user_client.snapshot_access_deny(snapshot_id, access['id'])
        self.user_client.wait_for_access_rule_deletion(
            snapshot_id, access['id'], is_snapshot=True)

        self.assertRaises(tempest_lib_exc.NotFound,
                          self.user_client.get_access, snapshot_id,
                          access['id'], is_snapshot=True)

    def test_create_delete_snapshot_ip_access_rule(self):
        snapshot = self.create_snapshot(share=self.share['id'],
                                        client=self.get_user_client(),
                                        cleanup_in_class=False)
        self._create_delete_access_rule(
            snapshot['id'], 'ip', self.access_to['ip'][0])

    def test_create_delete_snapshot_user_access_rule(self):
        snapshot = self.create_snapshot(share=self.share['id'],
                                        client=self.get_user_client(),
                                        cleanup_in_class=False)
        self._create_delete_access_rule(
            snapshot['id'], 'user', CONF.username_for_user_rules)

    def test_create_delete_snapshot_cert_access_rule(self):
        snapshot = self.create_snapshot(share=self.share['id'],
                                        client=self.get_user_client(),
                                        cleanup_in_class=False)
        self._create_delete_access_rule(
            snapshot['id'], 'cert', self.access_to['cert'][0])


@testtools.skipUnless(CONF.run_snapshot_tests and
                      CONF.run_mount_snapshot_tests,
                      "Snapshots or mountable snapshots tests are disabled.")
class NFSSnapshotAccessTest(SnapshotAccessReadBase):
    protocol = 'nfs'


@testtools.skipUnless(CONF.run_snapshot_tests and
                      CONF.run_mount_snapshot_tests,
                      "Snapshots or mountable snapshots tests are disabled.")
class CIFSSnapshotAccessTest(SnapshotAccessReadBase):
    protocol = 'cifs'


@testtools.skipUnless(CONF.run_snapshot_tests and
                      CONF.run_mount_snapshot_tests,
                      "Snapshots or mountable snapshots tests are disabled.")
class GlusterFSSnapshotAccessTest(SnapshotAccessReadBase):
    protocol = 'glusterfs'


@testtools.skipUnless(CONF.run_snapshot_tests and
                      CONF.run_mount_snapshot_tests,
                      "Snapshots or mountable snapshots tests are disabled.")
class HDFSSnapshotAccessTest(SnapshotAccessReadBase):
    protocol = 'hdfs'


@testtools.skipUnless(CONF.run_snapshot_tests and
                      CONF.run_mount_snapshot_tests,
                      "Snapshots or mountable snapshots tests are disabled.")
class MAPRFSSnapshotAccessTest(SnapshotAccessReadBase):
    protocol = 'maprfs'


def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if type(test_case._tests[0]) is SnapshotAccessReadBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



# Copyright 2016 Huawei inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient import exceptions
from manilaclient import extension
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_snapshot_instances


extensions = [
    extension.Extension('share_snapshot_instances', share_snapshot_instances),
]
cs = fakes.FakeClient(extensions=extensions)


@ddt.ddt
class SnapshotInstancesTest(utils.TestCase):

    def setUp(self):
        super(SnapshotInstancesTest, self).setUp()
        microversion = api_versions.APIVersion("2.19")
        mock_microversion = mock.Mock(api_version=microversion)
        self.manager = share_snapshot_instances.ShareSnapshotInstanceManager(
            api=mock_microversion)

    @ddt.data(True, False)
    def test_list(self, detailed):
        if detailed:
            url = '/snapshot-instances/detail'
        else:
            url = '/snapshot-instances'
        self.mock_object(self.manager, '_list', mock.Mock())
        self.manager.list(detailed=detailed, search_opts=None)
        self.manager._list.assert_called_once_with(url, 'snapshot_instances')

    @ddt.data(True, False)
    def test_list_with_snapshot(self, detailed):
        if detailed:
            url = '/snapshot-instances/detail'
        else:
            url = '/snapshot-instances'
        self.mock_object(self.manager, '_list', mock.Mock())
        self.manager.list(detailed=detailed, snapshot='snapshot_id')
        self.manager._list.assert_called_once_with(
            (url + '?snapshot_id=snapshot_id'), 'snapshot_instances',)

    def test_get(self):
        self.mock_object(self.manager, '_get', mock.Mock())
        self.manager.get('fake_snapshot_instance')
        self.manager._get.assert_called_once_with(
            '/snapshot-instances/' + 'fake_snapshot_instance',
            'snapshot_instance')

    def test_reset_instance_state(self):
        state = 'available'

        self.mock_object(self.manager, '_action', mock.Mock())
        self.manager.reset_state('fake_instance', state)
        self.manager._action.assert_called_once_with(
            "reset_status", 'fake_instance', {"status": state})

    @ddt.data('get', 'list', 'reset_state')
    def test_upsupported_microversion(self, method_name):
        unsupported_microversions = ('1.0', '2.18')
        arguments = {
            'instance': 'FAKE_INSTANCE',
        }
        if method_name in ('list'):
            arguments.clear()

        for microversion in unsupported_microversions:
            microversion = api_versions.APIVersion(microversion)
            mock_microversion = mock.Mock(api_version=microversion)
            manager = share_snapshot_instances.ShareSnapshotInstanceManager(
                api=mock_microversion)
            method = getattr(manager, method_name)
            self.assertRaises(exceptions.UnsupportedVersion,
                              method, **arguments)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from manilaclient import base
from manilaclient import exceptions
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import shares


cs = fakes.FakeClient()


class BaseTest(utils.TestCase):

    def test_resource_repr(self):
        r = base.Resource(None, dict(foo="bar", baz="spam"))
        self.assertEqual(repr(r), "<Resource baz=spam, foo=bar>")

    def test_eq(self):
        # Two resources of the same type with the same id: equal
        # The truth of r1==r2 does not imply that r1!=r2 is false in PY2.
        # Test that inequality operator is defined and that comparing equal
        # items returns False.
        r1 = base.Resource(None, {'id': 1, 'name': 'hi'})
        r2 = base.Resource(None, {'id': 1, 'name': 'hello'})
        self.assertTrue(r1 == r2)
        self.assertFalse(r1 != r2)

        # Two resources of different types: never equal
        r1 = base.Resource(None, {'id': 1})
        r2 = shares.Share(None, {'id': 1})
        self.assertNotEqual(r1, r2)
        self.assertTrue(r1 != r2)

        # Two resources with no ID: equal if their info is equal
        # The truth of r1==r2 does not imply that r1!=r2 is false in PY2.
        # Test that inequality operator is defined and that comparing equal
        # items returns False.
        r1 = base.Resource(None, {'name': 'joe', 'age': 12})
        r2 = base.Resource(None, {'name': 'joe', 'age': 12})
        self.assertTrue(r1 == r2)
        self.assertFalse(r1 != r2)

    def test_findall_invalid_attribute(self):
        # Make sure findall with an invalid attribute doesn't cause errors.
        # The following should not raise an exception.
        cs.shares.findall(vegetable='carrot')

        # However, find() should raise an error
        self.assertRaises(exceptions.NotFound,
                          cs.shares.find,
                          vegetable='carrot')

    def test_findall_with_all_tenants(self):
        cs.shares.list = mock.Mock(return_value=[])
        cs.shares.findall()
        cs.shares.list.assert_called_with(
            search_opts={'all_tenants': 1, 'is_soft_deleted': True})



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
import warnings

from manilaclient.v2.contrib import list_extensions

warnings.warn(
    "Module manilaclient.v1.contrib.list_extensions is deprecated "
    "(taken as a basis for manilaclient.v2.contrib.list_extensions). "
    "The preferable way to get a client class or object is to use "
    "the manilaclient.client module.")


class MovedModule(object):
    def __init__(self, new_module):
        self.new_module = new_module

    def __getattr__(self, attr):
        return getattr(self.new_module, attr)


sys.modules["manilaclient.v1.contrib.list_extensions"] = MovedModule(
    list_extensions)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient import client
from manilaclient import exceptions
from manilaclient.tests.unit import utils
import manilaclient.v1.client
import manilaclient.v2.client


@ddt.ddt
class ClientTest(utils.TestCase):

    def test_get_client_class_v2(self):
        output = manilaclient.client.get_client_class('2')
        self.assertEqual(output, manilaclient.v2.client.Client)

    def test_get_client_class_unknown(self):
        self.assertRaises(manilaclient.exceptions.UnsupportedVersion,
                          manilaclient.client.get_client_class, '0')

    @ddt.data('1', '1.0')
    def test_init_client_with_string_v1_version(self, version):
        with mock.patch.object(manilaclient.v1.client, 'Client'):
            with mock.patch.object(api_versions, 'APIVersion'):
                api_instance = api_versions.APIVersion.return_value
                api_instance.get_major_version.return_value = '1'

                manilaclient.client.Client(version, 'foo', bar='quuz')

                manilaclient.v1.client.Client.assert_called_once_with(
                    'foo', api_version=api_instance, bar='quuz')
                api_versions.APIVersion.assert_called_once_with('1.0')

    @ddt.data(
        ('2', '2.0'),
        ('2.0', '2.0'),
        ('2.6', '2.6'),
    )
    @ddt.unpack
    def test_init_client_with_string_v2_version(self, provided, expected):
        with mock.patch.object(manilaclient.v2.client, 'Client'):
            with mock.patch.object(api_versions, 'APIVersion'):
                api_instance = api_versions.APIVersion.return_value
                api_instance.get_major_version.return_value = '2'

                manilaclient.client.Client(provided, 'foo', bar='quuz')

                manilaclient.v2.client.Client.assert_called_once_with(
                    'foo', api_version=api_instance, bar='quuz')
                api_versions.APIVersion.assert_called_once_with(expected)

    def test_init_client_with_api_version_instance(self):
        version = manilaclient.API_MAX_VERSION
        with mock.patch.object(manilaclient.v2.client, 'Client'):

            manilaclient.client.Client(version, 'foo', bar='quuz')

            manilaclient.v2.client.Client.assert_called_once_with(
                'foo', api_version=version, bar='quuz')

    @ddt.data(None, '', '3', 'v1', 'v2', 'v1.0', 'v2.0')
    def test_init_client_with_unsupported_version(self, v):
        self.assertRaises(exceptions.UnsupportedVersion, client.Client, v)

    @ddt.data(
        ('1', '1.0'),
        ('1', '2.0'),
        ('1', '2.7'),
        ('1', None),
        ('1.0', '1.0'),
        ('1.0', '2.0'),
        ('1.0', '2.7'),
        ('1.0', None),
        ('2', '1.0'),
        ('2', '2.0'),
        ('2', '2.7'),
        ('2', None),
    )
    @ddt.unpack
    def test_init_client_with_version_parms(self, pos, kw):

        major = int(float(pos))
        pos_av = mock.Mock()
        kw_av = mock.Mock()

        with mock.patch.object(manilaclient.v1.client, 'Client'):
            with mock.patch.object(manilaclient.v2.client, 'Client'):
                with mock.patch.object(api_versions, 'APIVersion'):
                    api_versions.APIVersion.side_effect = [pos_av, kw_av]
                    pos_av.get_major_version.return_value = str(major)

                    if kw is None:
                        manilaclient.client.Client(pos, 'foo')
                        expected_av = pos_av
                    else:
                        manilaclient.client.Client(pos, 'foo', api_version=kw)
                        expected_av = kw_av

                    if int(float(pos)) == 1:
                        expected_client_ver = api_versions.DEPRECATED_VERSION
                        self.assertFalse(manilaclient.v2.client.Client.called)
                        manilaclient.v1.client.Client.assert_has_calls([
                            mock.call('foo', api_version=expected_av)
                        ])
                    else:
                        expected_client_ver = api_versions.MIN_VERSION
                        self.assertFalse(manilaclient.v1.client.Client.called)
                        manilaclient.v2.client.Client.assert_has_calls([
                            mock.call('foo', api_version=expected_av)
                        ])

                    if kw is None:
                        api_versions.APIVersion.assert_called_once_with(
                            expected_client_ver)
                    else:
                        api_versions.APIVersion.assert_has_calls([
                            mock.call(expected_client_ver),
                            mock.call(kw),
                        ])



# Copyright 2019 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.


"""OpenStackClient plugin for the Shared File System Service."""

import logging

from osc_lib import utils

from manilaclient import api_versions
from manilaclient.common import constants
from manilaclient import exceptions

LOG = logging.getLogger(__name__)

API_NAME = 'share'
API_VERSION_OPTION = 'os_share_api_version'
CLIENT_CLASS = 'manilaclient.v2.client.Client'
LATEST_VERSION = api_versions.MAX_VERSION
LATEST_MINOR_VERSION = api_versions.MAX_VERSION.split('.')[-1]


API_VERSIONS = {
    '2.%d' % i: CLIENT_CLASS
    for i in range(0, int(LATEST_MINOR_VERSION) + 1)
}


def _get_manila_url_from_service_catalog(instance):
    service_type = constants.SFS_SERVICE_TYPE
    url = instance.get_endpoint_for_service_type(
        constants.SFS_SERVICE_TYPE, region_name=instance._region_name,
        interface=instance.interface)
    # Fallback if cloud is using an older service type name
    if not url:
        url = instance.get_endpoint_for_service_type(
            constants.V2_SERVICE_TYPE, region_name=instance._region_name,
            interface=instance.interface)
        service_type = constants.V2_SERVICE_TYPE
    if url is None:
        raise exceptions.EndpointNotFound(
            message="Could not find manila / shared-file-system endpoint in "
                    "the service catalog.")
    return service_type, url


def make_client(instance):
    """Returns a shared file system service client."""
    requested_api_version = instance._api_version[API_NAME]

    shared_file_system_client = utils.get_client_class(
        API_NAME, requested_api_version, API_VERSIONS)

    # Cast the API version into an object for further processing
    requested_api_version = api_versions.APIVersion(
        version_str=requested_api_version)

    LOG.debug('Instantiating Shared File System (share) client: %s',
              shared_file_system_client)
    LOG.debug('Shared File System API version: %s',
              requested_api_version)

    service_type, manila_endpoint_url = _get_manila_url_from_service_catalog(
        instance)

    instance.setup_auth()
    debugging_enabled = instance._cli_options.debug
    client = shared_file_system_client(session=instance.session,
                                       service_catalog_url=manila_endpoint_url,
                                       endpoint_type=instance.interface,
                                       region_name=instance.region_name,
                                       service_type=service_type,
                                       auth=instance.auth,
                                       http_log_debug=debugging_enabled,
                                       api_version=requested_api_version,
                                       cacert=instance.cacert,
                                       cert=instance.cert,
                                       insecure=not instance.verify)
    return client


def build_option_parser(parser):
    """Hook to add global options."""
    default_api_version = utils.env('OS_SHARE_API_VERSION') or LATEST_VERSION
    parser.add_argument(
        '--os-share-api-version',
        metavar='<shared-file-system-api-version>',
        default=default_api_version,
        choices=sorted(
            API_VERSIONS,
            key=lambda k: [int(x) for x in k.split('.')]),
        help='Shared File System API version, default=' + default_api_version +
             'version supported by both the client and the server). '
             '(Env: OS_SHARE_API_VERSION)',
    )
    return parser



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt

from manilaclient.tests.functional import base


@ddt.ddt
class MessagesReadOnlyTest(base.BaseTestCase):

    @ddt.data(
        ("admin", "2.37"),
        ("user", "2.37"),
    )
    @ddt.unpack
    def test_message_list(self, role, microversion):
        self.skip_if_microversion_not_supported(microversion)
        self.clients[role].manila("message-list", microversion=microversion)


@ddt.ddt
class MessagesReadWriteTest(base.BaseTestCase):

    def setUp(self):
        super(MessagesReadWriteTest, self).setUp()
        self.message = self.create_message()

    def test_list_messages(self):
        self.skip_if_microversion_not_supported('2.37')
        messages = self.admin_client.list_messages()
        self.assertTrue(any(m['ID'] is not None for m in messages))
        self.assertTrue(any(m['User Message'] is not None for m in messages))
        self.assertTrue(any(m['Resource ID'] is not None for m in messages))
        self.assertTrue(any(m['Action ID'] is not None for m in messages))
        self.assertTrue(any(m['Detail ID'] is not None for m in messages))
        self.assertTrue(any(m['Resource Type'] is not None for m in messages))

    @ddt.data(
        'id', 'action_id', 'resource_id', 'action_id', 'detail_id',
        'resource_type', 'created_at', 'action_id,detail_id,resource_id',
    )
    def test_list_share_type_select_column(self, columns):
        self.skip_if_microversion_not_supported('2.37')
        self.admin_client.list_messages(columns=columns)

    def test_get_message(self):
        self.skip_if_microversion_not_supported('2.37')
        message = self.admin_client.get_message(self.message['ID'])
        expected_keys = (
            'id', 'action_id', 'resource_id', 'action_id', 'detail_id',
            'resource_type', 'created_at', 'created_at',
        )
        for key in expected_keys:
            self.assertIn(key, message)

    def test_delete_message(self):
        self.skip_if_microversion_not_supported('2.37')
        message = self.create_message(cleanup_in_class=False)
        self.admin_client.delete_message(message['ID'])
        self.admin_client.wait_for_message_deletion(message['ID'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import itertools
from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient import config
from manilaclient import exceptions
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_types

cs = fakes.FakeClient()

CONF = config.CONF

LATEST_MICROVERSION = CONF.max_api_microversion


def get_valid_type_create_data_2_0():

    public = [True, False]
    dhss = [True, False]
    snapshot = [None, True, False]
    extra_specs = [None, {'foo': 'bar'}]

    combos = list(itertools.product(public, dhss, snapshot, extra_specs))

    return combos


def get_valid_type_create_data_2_24():

    public = [True, False]
    dhss = [True, False]
    snapshot = [None]
    create_from_snapshot = [None]
    extra_specs = [None, {'replication_type': 'writable', 'foo': 'bar'}]

    snapshot_none_combos = list(itertools.product(public, dhss, snapshot,
                                                  create_from_snapshot,
                                                  extra_specs))

    public = [True, False]
    dhss = [True, False]
    snapshot = [True]
    create_from_snapshot = [True, False, None]
    extra_specs = [None, {'replication_type': 'readable', 'foo': 'bar'}]

    snapshot_true_combos = list(itertools.product(public, dhss, snapshot,
                                                  create_from_snapshot,
                                                  extra_specs))

    public = [True, False]
    dhss = [True, False]
    snapshot = [False]
    create_from_snapshot = [False, None]
    extra_specs = [None, {'replication_type': 'dr', 'foo': 'bar'}]

    snapshot_false_combos = list(itertools.product(public, dhss, snapshot,
                                                   create_from_snapshot,
                                                   extra_specs))

    return snapshot_none_combos + snapshot_true_combos + snapshot_false_combos


def get_valid_type_create_data_2_27():

    public = [True, False]
    dhss = [True, False]
    snapshot = [None]
    create_from_snapshot = [None]
    revert_to_snapshot = [None]
    extra_specs = [None, {'replication_type': 'writable', 'foo': 'bar'}]

    snapshot_none_combos = list(itertools.product(public, dhss, snapshot,
                                                  create_from_snapshot,
                                                  revert_to_snapshot,
                                                  extra_specs))

    public = [True, False]
    dhss = [True, False]
    snapshot = [True]
    create_from_snapshot = [True, False, None]
    revert_to_snapshot = [True, False, None]
    extra_specs = [None, {'replication_type': 'readable', 'foo': 'bar'}]

    snapshot_true_combos = list(itertools.product(public, dhss, snapshot,
                                                  create_from_snapshot,
                                                  revert_to_snapshot,
                                                  extra_specs))

    public = [True, False]
    dhss = [True, False]
    snapshot = [False]
    create_from_snapshot = [False, None]
    revert_to_snapshot = [False, None]
    extra_specs = [None, {'replication_type': 'dr', 'foo': 'bar'}]

    snapshot_false_combos = list(itertools.product(public, dhss, snapshot,
                                                   create_from_snapshot,
                                                   revert_to_snapshot,
                                                   extra_specs))

    return snapshot_none_combos + snapshot_true_combos + snapshot_false_combos


@ddt.ddt
class TypesTest(utils.TestCase):

    def _get_share_types_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return share_types.ShareTypeManager(api=mock_microversion)

    @ddt.data(
        {'snapshot_support': 'False'},
        {'snapshot_support': 'False', 'foo': 'bar'},
    )
    def test_init(self, extra_specs):
        info = {'extra_specs': extra_specs}

        share_type = share_types.ShareType(share_types.ShareTypeManager, info)

        self.assertTrue(hasattr(share_type, '_required_extra_specs'))
        self.assertTrue(hasattr(share_type, '_optional_extra_specs'))
        self.assertIsInstance(share_type._required_extra_specs, dict)
        self.assertIsInstance(share_type._optional_extra_specs, dict)
        self.assertEqual(extra_specs, share_type.get_optional_keys())

    def test_list_types(self):
        tl = cs.share_types.list()
        cs.assert_called('GET', '/types?is_public=all')
        for t in tl:
            self.assertIsInstance(t, share_types.ShareType)
            self.assertTrue(callable(getattr(t, 'get_required_keys', '')))
            self.assertTrue(callable(getattr(t, 'get_optional_keys', '')))
            self.assertEqual({'test': 'test'}, t.get_required_keys())
            self.assertEqual({'test1': 'test1'}, t.get_optional_keys())

    def test_list_types_only_public(self):
        cs.share_types.list(show_all=False)
        cs.assert_called('GET', '/types')

    def test_list_types_search_by_extra_specs(self):
        search_opts = {'extra_specs': {'aa': 'bb'}}
        cs.share_types.list(search_opts=search_opts)
        expect = '/types?extra_specs=%7B%27aa%27%3A+%27bb%27%7D&is_public=all'
        cs.assert_called('GET', expect)

    @ddt.data(*get_valid_type_create_data_2_0())
    @ddt.unpack
    def test_create_2_7(self, is_public, dhss, snapshot, extra_specs):

        extra_specs = copy.copy(extra_specs)

        manager = self._get_share_types_manager("2.7")
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        result = manager.create(
            'test-type-3', spec_driver_handles_share_servers=dhss,
            spec_snapshot_support=snapshot, extra_specs=extra_specs,
            is_public=is_public)

        if extra_specs is None:
            extra_specs = {}

        expected_extra_specs = dict(extra_specs)

        expected_body = {
            "share_type": {
                "name": 'test-type-3',
                'share_type_access:is_public': is_public,
                "extra_specs": expected_extra_specs,
            }
        }

        expected_body["share_type"]["extra_specs"][
            "driver_handles_share_servers"] = dhss
        expected_body["share_type"]["extra_specs"]['snapshot_support'] = (
            True if snapshot is None else snapshot)

        manager._create.assert_called_once_with(
            "/types", expected_body, "share_type")
        self.assertEqual("fake", result)

    def _add_standard_extra_specs_to_dict(self, extra_specs,
                                          create_from_snapshot=None,
                                          revert_to_snapshot=None,
                                          mount_snapshot=None):

        # Short-circuit checks to allow for extra specs to be (and remain) None
        if all(spec is None for spec in [
                create_from_snapshot, revert_to_snapshot, mount_snapshot]):
            return extra_specs

        extra_specs = extra_specs or {}

        if create_from_snapshot is not None:
            extra_specs['create_share_from_snapshot_support'] = (
                create_from_snapshot)
        if revert_to_snapshot is not None:
            extra_specs['revert_to_snapshot_support'] = (
                revert_to_snapshot)
        if mount_snapshot is not None:
            extra_specs['mount_snapshot_support'] = (
                mount_snapshot)

        return extra_specs

    @ddt.data(*get_valid_type_create_data_2_24())
    @ddt.unpack
    def test_create_2_24(self, is_public, dhss, snapshot, create_from_snapshot,
                         extra_specs):

        extra_specs = copy.copy(extra_specs)
        extra_specs = self._add_standard_extra_specs_to_dict(
            extra_specs, create_from_snapshot=create_from_snapshot)

        manager = self._get_share_types_manager("2.24")
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        result = manager.create(
            'test-type-3', spec_driver_handles_share_servers=dhss,
            spec_snapshot_support=snapshot,
            extra_specs=extra_specs, is_public=is_public)

        expected_extra_specs = dict(extra_specs or {})
        expected_extra_specs["driver_handles_share_servers"] = dhss

        if snapshot is None:
            expected_extra_specs.pop("snapshot_support", None)
        else:
            expected_extra_specs["snapshot_support"] = snapshot

        if create_from_snapshot is None:
            expected_extra_specs.pop("create_share_from_snapshot_support",
                                     None)
        else:
            expected_extra_specs["create_share_from_snapshot_support"] = (
                create_from_snapshot)

        expected_body = {
            "share_type": {
                "name": 'test-type-3',
                'share_type_access:is_public': is_public,
                "extra_specs": expected_extra_specs,
            }
        }

        manager._create.assert_called_once_with(
            "/types", expected_body, "share_type")
        self.assertEqual("fake", result)

    @ddt.data(*get_valid_type_create_data_2_27())
    @ddt.unpack
    def test_create_2_27(self, is_public, dhss, snapshot, create_from_snapshot,
                         revert_to_snapshot, extra_specs):

        extra_specs = copy.copy(extra_specs)
        extra_specs = self._add_standard_extra_specs_to_dict(
            extra_specs, create_from_snapshot=create_from_snapshot,
            revert_to_snapshot=revert_to_snapshot)

        manager = self._get_share_types_manager("2.27")
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        result = manager.create(
            'test-type-3', spec_driver_handles_share_servers=dhss,
            spec_snapshot_support=snapshot,
            extra_specs=extra_specs, is_public=is_public)

        expected_extra_specs = dict(extra_specs or {})
        expected_extra_specs["driver_handles_share_servers"] = dhss

        if snapshot is None:
            expected_extra_specs.pop("snapshot_support", None)
        else:
            expected_extra_specs["snapshot_support"] = snapshot

        if create_from_snapshot is None:
            expected_extra_specs.pop("create_share_from_snapshot_support",
                                     None)
        else:
            expected_extra_specs["create_share_from_snapshot_support"] = (
                create_from_snapshot)

        if revert_to_snapshot is None:
            expected_extra_specs.pop("revert_to_snapshot_support", None)
        else:
            expected_extra_specs["revert_to_snapshot_support"] = (
                revert_to_snapshot)

        expected_body = {
            "share_type": {
                "name": 'test-type-3',
                'share_type_access:is_public': is_public,
                "extra_specs": expected_extra_specs,
            }
        }

        manager._create.assert_called_once_with(
            "/types", expected_body, "share_type")
        self.assertEqual("fake", result)

    @ddt.data(
        (False, False, True, {'snapshot_support': True,
                              'replication_type': 'fake_repl_type'}),
        (False, False, False, {'snapshot_support': False,
                               'replication_type': 'fake_repl_type'}),
        (False, False, True, {'snapshot_support': False,
                              'replication_type': 'fake_repl_type'}),
        (False, False, False, {'snapshot_support': True,
                               'replication_type': 'fake_repl_type'}),

        (False, True, None, {'driver_handles_share_servers': True}),
        (False, False, None, {'driver_handles_share_servers': True}),
        (False, None, None, {'driver_handles_share_servers': True}),
        (False, None, None, {'driver_handles_share_servers': None}),
    )
    @ddt.unpack
    def test_create_error_2_7(self, is_public, dhss, snapshot,
                              extra_specs):
        manager = self._get_share_types_manager("2.7")
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        self.assertRaises(
            exceptions.CommandError,
            manager.create,
            'test-type-3',
            spec_driver_handles_share_servers=dhss,
            spec_snapshot_support=snapshot,
            extra_specs=extra_specs,
            is_public=is_public)

    @ddt.data(
        (False, True, None, None, {'driver_handles_share_servers': True}),
        (False, False, False, False, {'snapshot_support': True,
                                      'replication_type': 'fake_repl_type'}),
    )
    @ddt.unpack
    def test_create_error_2_24(self, is_public, dhss, snapshot,
                               create_from_snapshot, extra_specs):

        extra_specs = copy.copy(extra_specs)
        extra_specs = self._add_standard_extra_specs_to_dict(
            extra_specs, create_from_snapshot=create_from_snapshot)

        manager = self._get_share_types_manager("2.24")
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        self.assertRaises(
            exceptions.CommandError,
            manager.create,
            'test-type-3',
            spec_driver_handles_share_servers=dhss,
            spec_snapshot_support=snapshot,
            extra_specs=extra_specs,
            is_public=is_public)

    @ddt.data(
        ("2.6", True),
        ("2.7", True),
        ("2.24", True),
        ("2.41", True),
        ("2.6", False),
        ("2.7", False),
        ("2.24", False),
        ("2.41", False),
    )
    @ddt.unpack
    def test_create_with_default_values(self, microversion, dhss):

        manager = self._get_share_types_manager(microversion)
        self.mock_object(manager, '_create', mock.Mock(return_value="fake"))

        description = 'test description'
        if (api_versions.APIVersion(microversion) >=
                api_versions.APIVersion("2.41")):
            result = manager.create(
                'test-type-3', dhss, description=description)
        else:
            result = manager.create('test-type-3', dhss)

        if (api_versions.APIVersion(microversion) >
                api_versions.APIVersion("2.6")):
            is_public_keyname = "share_type_access:is_public"
        else:
            is_public_keyname = "os-share-type-access:is_public"

        expected_body = {
            "share_type": {
                "name": 'test-type-3',
                is_public_keyname: True,
                "extra_specs": {
                    "driver_handles_share_servers": dhss,
                    "snapshot_support": True,
                }
            }
        }

        if (api_versions.APIVersion(microversion) >=
                api_versions.APIVersion("2.24")):
            del expected_body['share_type']['extra_specs']['snapshot_support']

        if (api_versions.APIVersion(microversion) >=
                api_versions.APIVersion("2.41")):
            expected_body['share_type']['description'] = description
        manager._create.assert_called_once_with(
            "/types", expected_body, "share_type")
        self.assertEqual("fake", result)

    def test_set_key(self):
        t = cs.share_types.get(1)
        t.set_keys({'k': 'v'})
        cs.assert_called('POST',
                         '/types/1/extra_specs',
                         {'extra_specs': {'k': 'v'}})

    def test_unset_keys(self):
        t = cs.share_types.get(1)
        t.unset_keys(['k'])
        cs.assert_called('DELETE', '/types/1/extra_specs/k')

    @ddt.data(*set(('2.50', LATEST_MICROVERSION)))
    def test_update(self, microversion):
        manager = self._get_share_types_manager(microversion)
        self.mock_object(manager, '_update', mock.Mock(return_value="fake"))
        share_type = 1234
        name = "updated-test-type-1234"
        description = "updated test description"
        is_public_key_name = "share_type_access:is_public"
        is_public = False
        expected_body = {
            "share_type": {
                "name": name,
                is_public_key_name: is_public,
            }
        }
        result = manager.update(
            share_type, name, is_public, description)
        expected_body['share_type']['description'] = description
        manager._update.assert_called_once_with(
            "/types/%s" % share_type, expected_body, "share_type")
        self.assertEqual("fake", result)

    def test_delete(self):
        cs.share_types.delete(1)
        cs.assert_called('DELETE', '/types/1')

    def test_get_keys_from_resource_data(self):
        manager = mock.Mock()
        manager.api.client.get = mock.Mock(return_value=(200, {}))
        valid_extra_specs = {'test': 'test'}
        share_type = share_types.ShareType(mock.Mock(),
                                           {'extra_specs': valid_extra_specs,
                                            'name': 'test'},
                                           loaded=True)

        actual_result = share_type.get_keys()

        self.assertEqual(actual_result, valid_extra_specs)
        self.assertEqual(manager.api.client.get.call_count, 0)

    @ddt.data({'prefer_resource_data': True,
               'resource_extra_specs': {}},
              {'prefer_resource_data': False,
               'resource_extra_specs': {'fake': 'fake'}},
              {'prefer_resource_data': False,
              'resource_extra_specs': {}})
    @ddt.unpack
    def test_get_keys_from_api(self, prefer_resource_data,
                               resource_extra_specs):
        manager = mock.Mock()
        valid_extra_specs = {'test': 'test'}
        manager.api.client.get = mock.Mock(
            return_value=(200, {'extra_specs': valid_extra_specs}))
        info = {
            'name': 'test',
            'uuid': 'fake',
            'extra_specs': resource_extra_specs
        }
        share_type = share_types.ShareType(manager, info, loaded=True)

        actual_result = share_type.get_keys(prefer_resource_data)

        self.assertEqual(actual_result, valid_extra_specs)
        self.assertEqual(manager.api.client.get.call_count, 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from urllib import parse


class HookableMixin(object):
    """Mixin so classes can register and run hooks."""
    _hooks_map = {}

    @classmethod
    def add_hook(cls, hook_type, hook_func):
        if hook_type not in cls._hooks_map:
            cls._hooks_map[hook_type] = []

        cls._hooks_map[hook_type].append(hook_func)

    @classmethod
    def run_hooks(cls, hook_type, *args, **kwargs):
        hook_funcs = cls._hooks_map.get(hook_type) or []
        for hook_func in hook_funcs:
            hook_func(*args, **kwargs)


def safe_issubclass(*args):
    """Like issubclass, but will just return False if not a class."""

    try:
        if issubclass(*args):
            return True
    except TypeError:
        pass

    return False


def get_function_name(func):
    return "%s.%s" % (func.__module__, func.__qualname__)


def safe_urlencode(params_dict):
    """Workaround incompatible change to urllib.parse

    urllib's parse library used to adhere to RFC 2396 until
    python 3.7. The library moved from RFC 2396 to RFC 3986
    for quoting URL strings in python 3.7 and '~' is now
    included in the set of reserved characters. [1]

    This utility ensures "~" is never encoded.

    See LP 1785283 [2] for more details.
    [1] https://docs.python.org/3/library/urllib.parse.html#url-quoting
    [2] https://bugs.launchpad.net/python-manilaclient/+bug/1785283

    :param params_dict can be a list of (k,v) tuples, or a dictionary
    """

    parsed_params = parse.urlencode(params_dict)
    return parsed_params.replace("%7E", "~")



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.


from osc_lib.command import command
from osc_lib import utils as oscutils

from manilaclient.common._i18n import _


class ShareAvailabilityZoneList(command.Lister):
    """List all availability zones."""

    _description = _("List all availability zones")

    def get_parser(self, prog_name):
        parser = super(ShareAvailabilityZoneList, self).get_parser(
            prog_name)
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        availability_zones = share_client.availability_zones.list()

        fields = ("Id", "Name", "Created At", "Updated At")

        return (fields, (oscutils.get_item_properties
                (s, fields) for s in availability_zones))



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from manilaclient.tests.functional.osc import base
from tempest.lib.common.utils import data_utils


class SharesCLITest(base.OSCClientTestBase):

    def test_openstack_share_create(self):
        share_name = 'test_create_share'
        share = self.create_share(name=share_name)

        self.assertEqual(share['share_proto'], 'NFS')
        self.assertEqual(share['size'], '1')
        self.assertEqual(share['name'], share_name)

        shares_list = self.listing_result('share', 'list')
        self.assertIn(share['id'], [item['ID'] for item in shares_list])

    def test_openstack_share_list(self):
        share = self.create_share()
        shares_list = self.listing_result('share', 'list')
        self.assertTableStruct(shares_list, [
            'ID',
            'Name',
            'Size',
            'Share Proto',
            'Status',
            'Is Public',
            'Share Type Name',
            'Host',
            'Availability Zone'
        ])
        self.assertIn(share['id'], [item['ID'] for item in shares_list])

    def test_openstack_share_show(self):
        share = self.create_share()

        result = self.dict_result('share', 'show %s' % share['id'])
        self.assertEqual(share['id'], result['id'])

        listing_result = self.listing_result('share', 'show %s' % share['id'])
        self.assertTableStruct(listing_result, [
            'Field',
            'Value'
        ])

    def test_openstack_share_delete(self):
        share = self.create_share(add_cleanup=False)
        shares_list = self.listing_result('share', 'list')

        self.assertIn(share['id'], [item['ID'] for item in shares_list])

        self.openstack('share delete %s' % share['id'])
        self.check_object_deleted('share', share['id'])
        shares_list_after_delete = self.listing_result('share', 'list')

        self.assertNotIn(
            share['id'], [item['ID'] for item in shares_list_after_delete])

    def test_openstack_share_set(self):
        share = self.create_share()
        self.openstack(f'share set {share["id"]} --name new_name '
                       f'--property key=value')
        result = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual(share['id'], result['id'])
        self.assertEqual('new_name', result['name'])
        self.assertEqual("key='value'", result['properties'])

    def test_openstack_share_unset(self):
        share = self.create_share(name='test_name', properties={
            'foo': 'bar', 'test_key': 'test_value'})
        result1 = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual(share['id'], result1['id'])
        self.assertEqual(share['name'], result1['name'])
        self.assertEqual("foo='bar', test_key='test_value'",
                         result1['properties'])

        self.openstack(f'share unset {share["id"]} --name --property test_key')
        result2 = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual(share['id'], result2['id'])
        self.assertEqual('None', result2['name'])
        self.assertEqual("foo='bar'", result2['properties'])

    def test_openstack_share_soft_delete(self):
        share = self.create_share(name='test_share')
        result1 = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual(share['id'], result1['id'])
        self.assertEqual(share['name'], result1['name'])

        self.openstack(f'share delete {share["id"]} --soft')
        self.check_object_deleted('share', share['id'])
        shares_list_after_delete = self.listing_result('share', 'list '
                                                       '--soft-deleted')
        self.assertIn(
            share['id'], [item['ID'] for item in shares_list_after_delete])

    def test_openstack_share_resize(self):
        share = self.create_share()
        self.openstack(f'share resize {share["id"]} 10 --wait ')
        result = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual('10', result['size'])

    def test_openstack_share_revert(self):
        slug = "revert_test"
        share_type = self.create_share_type(
            name=data_utils.rand_name(slug),
            snapshot_support=True,
            revert_to_snapshot_support=True)
        share = self.create_share(share_type=share_type['id'], size=10)
        snapshot = self.create_snapshot(share['id'], wait=True)
        self.assertEqual(snapshot['size'], share['size'])
        self.openstack(f'share resize {share["id"]} 15 --wait')
        result1 = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual('15', result1["size"])

        self.openstack(f'share revert {snapshot["id"]} --wait')

        result2 = self.dict_result('share', f'show {share["id"]}')

        self.assertEqual('10', result2['size'])

    def test_openstack_share_abandon_adopt(self):
        share = self.create_share(add_cleanup=False)
        shares_list = self.listing_result('share', 'list')
        self.assertIn(share['id'], [item['ID'] for item in shares_list])
        export_location_obj = self.get_share_export_locations(share['id'])[0]
        export_location = export_location_obj['Path']
        source = self.dict_result('share', f'show {share["id"]}')
        host = source['host']
        protocol = source['share_proto']
        share_type = source['share_type']
        self.openstack(f'share abandon {share["id"]} --wait')

        # verify abandonded
        self.check_object_deleted('share', share['id'])
        shares_list_after_delete = self.listing_result('share', 'list')
        self.assertNotIn(
            share['id'], [item['ID'] for item in shares_list_after_delete])

        result = self.dict_result(
            'share', f'adopt {host} {protocol} {export_location} '
                     f'--share-type {share_type} --wait')

        # verify adopted
        self.assertEqual(host, result['host'])
        self.assertEqual(protocol, result['share_proto'])
        self.openstack(f'share delete {result["id"]} --wait')

    def test_openstack_share_export_location_show(self):
        share = self.create_share()
        share_export_locations = self.get_share_export_locations(share["id"])
        result_export_locations = self.listing_result(
            'share', f'export location list {share["id"]}')
        for share_export in share_export_locations:
            export_location = self.dict_result(
                'share', f'export location show {share["id"]} '
                         f'{share_export["ID"]}')
            self.assertIn(export_location["id"], [item["ID"] for item in
                          result_export_locations])

    def test_openstack_share_export_location_list(self):
        share = self.create_share()
        share_export_locations = self.get_share_export_locations(share["id"])
        result_export_locations = self.listing_result(
            'share', f'export location list {share["id"]}')

        self.assertTableStruct(result_export_locations, [
            'ID',
            'Path'
        ])
        export_location_ids = [el['ID'] for el in share_export_locations]
        for share_export in result_export_locations:
            self.assertIn(share_export["ID"], export_location_ids)

    def test_openstack_share_restore(self):
        share = self.create_share(name='test_share')
        result1 = self.dict_result('share', f'show {share["id"]}')
        self.assertEqual(share['id'], result1['id'])
        self.assertEqual(share['name'], result1['name'])

        self.openstack(f'share delete {share["id"]} --soft')
        self.check_object_deleted('share', share['id'])
        shares_list_after_delete = self.listing_result('share', 'list')
        self.assertNotIn(
            share['id'], [item['ID'] for item in shares_list_after_delete])

        self.openstack(f'share restore {share["id"]} ')

        shares_list_after_restore = self.listing_result('share', 'list')
        self.assertIn(
            share['id'], [item['ID'] for item in shares_list_after_restore])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient.tests.functional.osc import base


class ShareServicesTestCase(base.OSCClientTestBase):

    def test_services_list(self):
        services = self.list_services()
        self.assertTableStruct(services, [
            'ID',
            'Binary',
            'Host',
            'Zone',
            'Status',
            'State',
            'Updated At'
        ])
        self.assertTrue(len(services) > 0)

        # Filter results
        first_service = services[0]

        services = self.list_services(host=first_service['Host'],
                                      status=first_service['Status'],
                                      state=first_service['State'])
        self.assertEqual(1, len(services))
        for attr in ('ID', 'Binary', 'Host', 'State', 'Status', 'Zone'):
            self.assertEqual(first_service[attr], services[0][attr])

    def test_services_set(self):
        services = self.list_services()
        service = [service for service in services if
                   service["Binary"] == "manila-data"]
        first_service = service[0]
        self.openstack(f'share service set {first_service["Host"]} '
                       f'{first_service["Binary"]} '
                       '--disable')
        result = self.listing_result('share service',
                                     'list --status disabled')
        self.assertEqual(first_service['ID'], result[0]['ID'])
        self.assertEqual('disabled', result[0]['Status'])

        # enable the share service again
        self.openstack(f'share service set {first_service["Host"]} '
                       f'{first_service["Binary"]} '
                       '--enable')



# Copyright 2021 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from osc_lib.command import command
from osc_lib import utils as osc_utils

from manilaclient.common._i18n import _


class ShareInstanceListExportLocation(command.Lister):
    """List share instance export locations."""
    _description = _("List share instance export locations")

    def get_parser(self, prog_name):
        parser = super(
            ShareInstanceListExportLocation, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("ID of the share instance.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        instance = osc_utils.find_resource(
            share_client.share_instances,
            parsed_args.instance)

        export_locations = share_client.share_instance_export_locations.list(
            instance,
            search_opts=None)

        columns = [
            'ID',
            'Path',
            'Is Admin Only',
            'Preferred',
        ]

        data = (
            osc_utils.get_dict_properties(export_location._info, columns)
            for export_location in export_locations
        )

        return (columns, data)


class ShareInstanceShowExportLocation(command.ShowOne):
    """Display the export location for a share instance."""
    _description = _(
        "Show export location for a share instance.")

    def get_parser(self, prog_name):
        parser = super(
            ShareInstanceShowExportLocation, self).get_parser(prog_name)
        parser.add_argument(
            "instance",
            metavar="<instance>",
            help=_("Name or ID of the share instance")
        )
        parser.add_argument(
            "export_location",
            metavar="<export_location>",
            help=_("ID of the share instance export location.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        share_instance = osc_utils.find_resource(
            share_client.share_instances,
            parsed_args.instance)

        share_instance_export_locations = (
            share_client.share_instance_export_locations.get(
                share_instance.id,
                parsed_args.export_location)
        )

        data = share_instance_export_locations._info
        data.pop('links', None)

        return self.dict2columns(data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils as osc_utils

from manilaclient.common._i18n import _


class SetShareService(command.Command):
    """Enable/disable share service (Admin only)."""
    _description = _("Enable/Disable share service (Admin only).")

    def get_parser(self, prog_name):
        parser = super(SetShareService, self).get_parser(prog_name)
        parser.add_argument(
            'host',
            metavar='<host>',
            help=_("Host name as 'example_host@example_backend'.")
        )
        parser.add_argument(
            'binary',
            metavar='<binary>',
            help=_("Service binary, could be 'manila-share', "
                   "'manila-scheduler' or 'manila-data'")
        )
        enable_group = parser.add_mutually_exclusive_group()
        enable_group.add_argument(
            '--enable',
            action='store_true',
            help=_('Enable share service'),
        )
        enable_group.add_argument(
            '--disable',
            action='store_true',
            help=_('Disable share service'),
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        if parsed_args.enable:
            try:
                share_client.services.enable(
                    parsed_args.host, parsed_args.binary)
            except Exception as e:
                raise exceptions.CommandError(_(
                    "Failed to enable service: %s" % e))

        if parsed_args.disable:
            try:
                share_client.services.disable(
                    parsed_args.host, parsed_args.binary)
            except Exception as e:
                raise exceptions.CommandError(_(
                    "Failed to disable service: %s" % e))


class ListShareService(command.Lister):
    """List share services (Admin only)."""
    _description = _("List share services (Admin only).")

    def get_parser(self, prog_name):
        parser = super(ListShareService, self).get_parser(prog_name)
        parser.add_argument(
            "--host",
            metavar="<host>",
            default=None,
            help=_("Filter services by name of the host.")
        )
        parser.add_argument(
            "--binary",
            metavar="<binary>",
            default=None,
            help=_("Filter services by the name of the service.")
        )
        parser.add_argument(
            "--status",
            metavar="<status>",
            default=None,
            help=_("Filter results by status.")
        )
        parser.add_argument(
            "--state",
            metavar="<state>",
            default=None,
            choices=['up', 'down'],
            help=_("Filter results by state.")
        )
        parser.add_argument(
            "--zone",
            metavar="<zone>",
            default=None,
            help=_("Filter services by their availability zone.")
        )
        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        search_opts = {
            'host': parsed_args.host,
            'binary': parsed_args.binary,
            'status': parsed_args.status,
            'state': parsed_args.state,
            'zone': parsed_args.zone,
        }

        services = share_client.services.list(search_opts=search_opts)

        columns = [
            'ID',
            'Binary',
            'Host',
            'Zone',
            'Status',
            'State',
            'Updated At'
        ]

        data = (osc_utils.get_dict_properties(
            service._info, columns) for service in services)

        return (columns, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

from unittest import mock

from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.common.apiclient.exceptions import BadRequest
from manilaclient.common.apiclient.exceptions import NotFound
from manilaclient.osc import utils
from manilaclient.osc.v2 import share_group_types as osc_share_group_types
from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes

COLUMNS = [
    'ID',
    'Name',
    'Share Types',
    'Visibility',
    'Is Default',
    'Group Specs',
]


class TestShareGroupType(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareGroupType, self).setUp()

        self.sgt_mock = self.app.client_manager.share.share_group_types
        self.sgt_mock.reset_mock()
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)


class TestShareGroupTypeCreate(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeCreate, self).setUp()

        self.share_types = (
            manila_fakes.FakeShareType.create_share_types(count=2))

        formatted_share_types = []

        for st in self.share_types:
            formatted_share_types.append(st.name)

        self.share_group_type = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                attrs={
                    'share_types': formatted_share_types
                }
            ))

        self.share_group_type_formatted = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                attrs={
                    'id': self.share_group_type['id'],
                    'name': self.share_group_type['name'],
                    'share_types': formatted_share_types
                }
            ))

        formatted_sgt = utils.format_share_group_type(
            self.share_group_type_formatted)

        self.sgt_mock.create.return_value = self.share_group_type
        self.sgt_mock.get.return_value = self.share_group_type

        # Get the command object to test
        self.cmd = osc_share_group_types.CreateShareGroupType(self.app, None)

        self.data = tuple(formatted_sgt.values())
        self.columns = tuple(formatted_sgt.keys())

    def test_share_group_type_create_required_args(self):
        """Verifies required arguments."""

        arglist = [
            self.share_group_type.name,
            self.share_types[0].name,
            self.share_types[1].name,
        ]
        verifylist = [
            ('name', self.share_group_type.name),
            ('share_types', [self.share_types[0].name,
                             self.share_types[1].name])
        ]

        with mock.patch(
            'manilaclient.common.apiclient.utils.find_resource',
                side_effect=[self.share_types[0], self.share_types[1]]):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            columns, data = self.cmd.take_action(parsed_args)

            self.sgt_mock.create.assert_called_with(
                group_specs={},
                is_public=True,
                name=self.share_group_type.name,
                share_types=[
                    self.share_types[0].name, self.share_types[1].name]
            )

            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.data, data)

    def test_share_group_type_create_missing_required_arg(self):
        """Verifies missing required arguments."""

        arglist = [
            self.share_group_type.name,
        ]
        verifylist = [
            ('name', self.share_group_type.name)
        ]

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_group_type_create_private(self):
        arglist = [
            self.share_group_type.name,
            self.share_types[0].name,
            self.share_types[1].name,
            '--public', 'False'
        ]
        verifylist = [
            ('name', self.share_group_type.name),
            ('share_types', [self.share_types[0].name,
                             self.share_types[1].name]),
            ('public', 'False')
        ]

        with mock.patch(
            'manilaclient.common.apiclient.utils.find_resource',
            side_effect=[self.share_types[0],
                         self.share_types[1]]):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            columns, data = self.cmd.take_action(parsed_args)

            self.sgt_mock.create.assert_called_with(
                group_specs={},
                is_public=False,
                name=self.share_group_type.name,
                share_types=[self.share_types[0].name,
                             self.share_types[1].name]
            )

            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.data, data)

    def test_share_group_type_create_group_specs(self):

        arglist = [
            self.share_group_type.name,
            self.share_types[0].name,
            self.share_types[1].name,
            '--group-specs', 'consistent_snapshot_support=true'
        ]
        verifylist = [
            ('name', self.share_group_type.name),
            ('share_types', [self.share_types[0].name,
                             self.share_types[1].name]),
            ('group_specs', ['consistent_snapshot_support=true'])
        ]

        with mock.patch(
            'manilaclient.common.apiclient.utils.find_resource',
            side_effect=[self.share_types[0],
                         self.share_types[1]]):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)

            columns, data = self.cmd.take_action(parsed_args)

            self.sgt_mock.create.assert_called_with(
                group_specs={'consistent_snapshot_support': 'True'},
                is_public=True,
                name=self.share_group_type.name,
                share_types=[
                    self.share_types[0].name, self.share_types[1].name]
            )

            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.data, data)

    def test_create_share_group_type(self):
        arglist = [
            self.share_group_type.name,
            self.share_types[0].name,
            self.share_types[1].name
        ]
        verifylist = [
            ('name', self.share_group_type.name),
            ('share_types', [self.share_types[0].name,
                             self.share_types[1].name])
        ]

        with mock.patch(
            'manilaclient.common.apiclient.utils.find_resource',
            side_effect=[self.share_types[0],
                         self.share_types[1],
                         self.share_group_type]):
            parsed_args = self.check_parser(self.cmd, arglist, verifylist)
            columns, data = self.cmd.take_action(parsed_args)

            self.sgt_mock.create.assert_called_with(
                group_specs={},
                is_public=True,
                name=self.share_group_type.name,
                share_types=[self.share_types[0].name,
                             self.share_types[1].name]
            )

            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.data, data)


class TestShareGroupTypeDelete(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeDelete, self).setUp()

        self.share_group_types = (
            manila_fakes.FakeShareGroupType.create_share_group_types(count=2))

        self.sgt_mock.delete.return_value = None
        self.sgt_mock.get = (
            manila_fakes.FakeShareGroupType.get_share_group_types(
                self.share_group_types))

        # Get the command object to test
        self.cmd = osc_share_group_types.DeleteShareGroupType(self.app, None)

    def test_share_group_type_delete_one(self):
        arglist = [
            self.share_group_types[0].name
        ]

        verifylist = [
            ('share_group_types', [self.share_group_types[0].name])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)
        self.sgt_mock.delete.assert_called_with(
            self.share_group_types[0])
        self.assertIsNone(result)

    def test_share_group_type_delete_multiple(self):
        arglist = []
        for t in self.share_group_types:
            arglist.append(t.name)
        verifylist = [
            ('share_group_types', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        calls = []
        for t in self.share_group_types:
            calls.append(mock.call(t))
        self.sgt_mock.delete.assert_has_calls(calls)
        self.assertIsNone(result)

    def test_delete_share_group_type_with_exception(self):
        arglist = [
            'non_existing_type',
        ]
        verifylist = [
            ('share_group_types', arglist),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.sgt_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)

    def test_delete_share_group_type(self):
        arglist = [
            self.share_group_types[0].name
        ]

        verifylist = [
            ('share_group_types', [self.share_group_types[0].name])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.sgt_mock.delete.assert_called_with(self.share_group_types[0])

        self.assertIsNone(result)


class TestShareGroupTypeSet(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeSet, self).setUp()

        self.share_group_type = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                methods={'set_keys': None, 'update': None}))
        self.sgt_mock.get.return_value = self.share_group_type

        # Get the command object to test
        self.cmd = osc_share_group_types.SetShareGroupType(self.app, None)

    def test_share_group_type_set_group_specs(self):
        arglist = [
            self.share_group_type.id,
            '--group-specs', 'consistent_snapshot_support=true'
        ]
        verifylist = [
            ('share_group_type', self.share_group_type.id),
            ('group_specs', ['consistent_snapshot_support=true'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_group_type.set_keys.assert_called_with(
            {'consistent_snapshot_support': 'True'})
        self.assertIsNone(result)

    def test_share_group_type_set_extra_specs_exception(self):
        arglist = [
            self.share_group_type.id,
            '--group-specs', 'snapshot_support=true'
        ]
        verifylist = [
            ('share_group_type', self.share_group_type.id),
            ('group_specs', ['snapshot_support=true'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.share_group_type.set_keys.side_effect = BadRequest()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareGroupTypeUnset(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeUnset, self).setUp()

        self.share_group_type = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                methods={'unset_keys': None}))
        self.sgt_mock.get.return_value = self.share_group_type

        # Get the command object to test
        self.cmd = osc_share_group_types.UnsetShareGroupType(self.app, None)

    def test_share_group_type_unset_extra_specs(self):
        arglist = [
            self.share_group_type.id,
            'consistent_snapshot_support'
        ]
        verifylist = [
            ('share_group_type', self.share_group_type.id),
            ('group_specs', ['consistent_snapshot_support'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)
        self.share_group_type.unset_keys.assert_called_with(
            ['consistent_snapshot_support'])
        self.assertIsNone(result)

    def test_share_group_type_unset_exception(self):
        arglist = [
            self.share_group_type.id,
            'snapshot_support'
        ]
        verifylist = [
            ('share_group_type', self.share_group_type.id),
            ('group_specs', ['snapshot_support'])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.share_group_type.unset_keys.side_effect = NotFound()
        self.assertRaises(
            exceptions.CommandError, self.cmd.take_action, parsed_args)


class TestShareGroupTypeList(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeList, self).setUp()

        self.share_group_types = (
            manila_fakes.FakeShareGroupType.create_share_group_types())

        self.sgt_mock.list.return_value = self.share_group_types

        # Get the command object to test
        self.cmd = osc_share_group_types.ListShareGroupType(self.app, None)

        self.values = (oscutils.get_dict_properties(
            s._info, COLUMNS) for s in self.share_group_types)

    def test_share_group_type_list_no_options(self):
        arglist = []
        verifylist = [
            ('all', False)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.sgt_mock.list.assert_called_once_with(
            search_opts={},
            show_all=False
        )
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_group_type_list_all(self):
        arglist = [
            '--all',
        ]
        verifylist = [
            ('all', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.sgt_mock.list.assert_called_once_with(
            search_opts={},
            show_all=True)
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_group_type_list_group_specs(self):
        arglist = [
            '--group-specs', 'consistent_snapshot_support=true'
        ]
        verifylist = [
            ('group_specs', ['consistent_snapshot_support=true'])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.sgt_mock.list.assert_called_once_with(
            search_opts={'group_specs': {
                'consistent_snapshot_support': 'True'}},
            show_all=False)
        self.assertEqual(COLUMNS, columns)
        self.assertEqual(list(self.values), list(data))


class TestShareGroupTypeShow(TestShareGroupType):

    def setUp(self):
        super(TestShareGroupTypeShow, self).setUp()

        self.share_types = (
            manila_fakes.FakeShareType.create_share_types(count=2))

        formatted_share_types = []

        for st in self.share_types:
            formatted_share_types.append(st.name)

        self.share_group_type = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                attrs={
                    'share_types': formatted_share_types
                }
            ))

        self.share_group_type_formatted = (
            manila_fakes.FakeShareGroupType.create_one_share_group_type(
                attrs={
                    'id': self.share_group_type['id'],
                    'name': self.share_group_type['name'],
                    'share_types': formatted_share_types
                }
            ))

        formatted_sgt = utils.format_share_group_type(
            self.share_group_type_formatted)

        self.sgt_mock.get.return_value = self.share_group_type

        # Get the command object to test
        self.cmd = osc_share_group_types.ShowShareGroupType(self.app, None)

        self.data = tuple(formatted_sgt.values())
        self.columns = tuple(formatted_sgt.keys())

    def test_share_group_type_show(self):
        arglist = [
            self.share_group_type.name
        ]
        verifylist = [
            ("share_group_type", self.share_group_type.name)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        self.sgt_mock.get.assert_called_with(self.share_group_type)

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.


from oslo_serialization import jsonutils

from manilaclient.tests.functional.osc import base


class SharesGroupTypeAccessCLITest(base.OSCClientTestBase):

    def test_share_group_type_access_create(self):
        share_group_type_name = self.create_share_group_type(
            share_types='dhss_false', public=False)['name']

        access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')
        self.assertEqual(0, len(access_list))

        cmd_output = jsonutils.loads(self.openstack('token issue -f json '))
        auth_project_id = cmd_output['project_id']

        self.share_group_type_access_create(
            share_group_type_name, auth_project_id)

        share_group_type_access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')

        self.assertEqual(1, len(share_group_type_access_list))

    def test_share_group_type_access_delete(self):
        share_group_type_name = self.create_share_group_type(
            share_types='dhss_false', public=False)['name']

        access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')
        self.assertEqual(0, len(access_list))

        cmd_output = jsonutils.loads(self.openstack('token issue -f json '))
        auth_project_id = cmd_output['project_id']

        # Create using name to ensure that we are "translating" the
        # name into the actual ID in the CLI
        self.share_group_type_access_create(
            share_group_type_name, auth_project_id)

        share_group_type_access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')

        self.assertEqual(1, len(share_group_type_access_list))
        self.assertEqual(
            share_group_type_access_list[0]['Project ID'], auth_project_id)

        self.share_group_type_access_delete(
            share_group_type_name, auth_project_id)

        access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')
        self.assertEqual(0, len(access_list))

    def test_share_group_type_access_list(self):
        share_group_type_name = self.create_share_group_type(
            share_types='dhss_false', public=False)['name']

        access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')
        self.assertEqual(0, len(access_list))

        cmd_output = jsonutils.loads(self.openstack('token issue -f json '))
        auth_project_id = cmd_output['project_id']

        self.share_group_type_access_create(
            share_group_type_name, auth_project_id)

        share_group_type_access_list = self.listing_result(
            'share', f'group type access list {share_group_type_name}')

        self.assertEqual(1, len(share_group_type_access_list))
        self.assertTableStruct(access_list, ['Project ID'])



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Asynchronous User Message interface."""
from manilaclient import api_versions
from manilaclient import base
from manilaclient.common import constants

RESOURCES_PATH = '/messages'
RESOURCE_PATH = '/messages/%s'
RESOURCES_NAME = 'messages'
RESOURCE_NAME = 'message'


class Message(base.Resource):
    NAME_ATTR = 'id'

    def __repr__(self):
        return "<Message: %s>" % self.id

    def delete(self):
        """Delete this message."""
        return self.manager.delete(self)


class MessageManager(base.ManagerWithFind):
    """Manage :class:`Message` resources."""
    resource_class = Message

    @api_versions.wraps('2.37')
    def get(self, message_id):
        """Get a message.

        :param message_id: The ID of the message to get.
        :rtype: :class:`Message`
        """
        return self._get(RESOURCE_PATH % message_id, RESOURCE_NAME)

    @api_versions.wraps('2.37')
    def list(self, search_opts=None, sort_key=None, sort_dir=None):
        """Lists all messages.

        :param search_opts: Search options to filter out messages.
        :rtype: list of :class:`Message`
        """
        search_opts = search_opts or {}

        if sort_key is not None:
            if sort_key in constants.MESSAGE_SORT_KEY_VALUES:
                search_opts['sort_key'] = sort_key
            else:
                raise ValueError(
                    'sort_key must be one of the following: %s.'
                    % ', '.join(constants.MESSAGE_SORT_KEY_VALUES))

        if sort_dir is not None:
            if sort_dir in constants.SORT_DIR_VALUES:
                search_opts['sort_dir'] = sort_dir
            else:
                raise ValueError('sort_dir must be one of the following: %s.'
                                 % ', '.join(constants.SORT_DIR_VALUES))

        query_string = self._build_query_string(search_opts)

        path = RESOURCES_PATH + query_string
        return self._list(path, RESOURCES_NAME)

    @api_versions.wraps('2.37')
    def delete(self, message):
        """Delete a message."""

        loc = RESOURCE_PATH % base.getid(message)

        return self._delete(loc)



# Copyright 2013 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient.tests.unit import utils
from manilaclient.v2 import quota_classes


@ddt.ddt
class QuotaClassSetsTest(utils.TestCase):

    def _get_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return quota_classes.QuotaClassSetManager(api=mock_microversion)

    def _get_resource_path(self, microversion):
        if (api_versions.APIVersion(microversion) >
                api_versions.APIVersion("2.6")):
            return quota_classes.RESOURCE_PATH
        return quota_classes.RESOURCE_PATH_LEGACY

    @ddt.data("2.6", "2.7")
    def test_class_quotas_get(self, microversion):
        class_name = 'test'
        manager = self._get_manager(microversion)
        resource_path = self._get_resource_path(microversion)
        expected_url = "%s/test" % resource_path
        with mock.patch.object(manager, '_get',
                               mock.Mock(return_value='fake_get')):
            manager.get(class_name)

            manager._get.assert_called_once_with(
                expected_url, "quota_class_set")

    @ddt.data("2.6", "2.7")
    def test_update_quota(self, microversion):
        class_name = 'test'
        manager = self._get_manager(microversion)
        resource_path = self._get_resource_path(microversion)
        expected_url = "%s/test" % resource_path
        expected_body = {
            'quota_class_set': {
                'class_name': class_name,
                'shares': 1,
                'snapshots': 2,
                'gigabytes': 3,
                'snapshot_gigabytes': 4,
                'share_networks': 5,
            },
        }
        with mock.patch.object(manager, '_update',
                               mock.Mock(return_value='fake_update')):
            manager.update(
                class_name, shares=1, snapshots=2, gigabytes=3,
                snapshot_gigabytes=4, share_networks=5)

            manager._update.assert_called_once_with(
                expected_url, expected_body)



# Copyright 2019 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import logging

from oslo_utils import strutils

from manilaclient.common._i18n import _
from manilaclient.common import constants
from manilaclient import exceptions

LOG = logging.getLogger(__name__)


def extract_key_value_options(pairs):
    result_dict = {}
    duplicate_options = []
    pairs = pairs or {}

    for attr, value in pairs.items():
        if attr not in result_dict:
            result_dict[attr] = value
        else:
            duplicate_options.append(attr)

    if pairs and len(duplicate_options) > 0:
        duplicate_str = ', '.join(duplicate_options)
        msg = "Following options were duplicated: %s" % duplicate_str
        raise exceptions.CommandError(msg)

    return result_dict


def format_properties(properties):
    formatted_data = []

    for item in properties:
        formatted_data.append("%s : %s" % (item, properties[item]))
    return "\n".join(formatted_data)


def extract_properties(properties):
    result_dict = {}
    for item in properties:
        try:
            (key, value) = item.split('=', 1)
            if key in result_dict:
                raise exceptions.CommandError(
                    "Argument '%s' is specified twice." % key)
            else:
                result_dict[key] = value
        except ValueError:
            raise exceptions.CommandError(
                "Parsing error, expected format 'key=value' for " + item
            )
    return result_dict


def extract_extra_specs(extra_specs, specs_to_add,
                        bool_specs=constants.BOOL_SPECS):
    try:
        for item in specs_to_add:
            (key, value) = item.split('=', 1)
            if key in extra_specs:
                msg = ("Argument '%s' value specified twice." % key)
                raise exceptions.CommandError(msg)
            elif key in bool_specs:
                if strutils.is_valid_boolstr(value):
                    extra_specs[key] = value.capitalize()
                else:
                    msg = (
                        "Argument '%s' is of boolean "
                        "type and has invalid value: %s"
                        % (key, str(value)))
                    raise exceptions.CommandError(msg)
            else:
                extra_specs[key] = value
    except ValueError:
        msg = LOG.error(_(
            "Wrong format: specs should be key=value pairs."))
        raise exceptions.CommandError(msg)
    return extra_specs


def extract_group_specs(extra_specs, specs_to_add):
    return extract_extra_specs(extra_specs,
                               specs_to_add, constants.GROUP_BOOL_SPECS)


def format_column_headers(columns):
    column_headers = []
    for column in columns:
        column_headers.append(
            column.replace('_', ' ').title().replace('Id', 'ID'))
    return column_headers


def format_share_group_type(share_group_type, formatter='table'):
    printable_share_group_type = share_group_type._info

    is_public = printable_share_group_type.pop('is_public')

    printable_share_group_type['visibility'] = (
        'public' if is_public else 'private')

    if formatter == 'table':
        printable_share_group_type['group_specs'] = (
            format_properties(share_group_type.group_specs))
        printable_share_group_type['share_types'] = (
            "\n".join(printable_share_group_type['share_types'])
        )

    return printable_share_group_type
